
import logging
import logging.config
import os
import queue
import threading
import copy
import time
from logging.handlers import QueueHandler, QueueListener

from .utils import get_log_default_path, get_digit_from_env, get_log_given_path


class Logger:

    LogHandlerSample = {
        'console': {
            'level': None,
            'class': 'logging.StreamHandler',
            'formatter': 'standard'
        },
        'file': {
            'level': None,
            'class': None,
            'filename': None,
            'formatter': 'standard',
            'encoding': 'utf-8',
            'backupCount': None
        }
     }

    LogConfigLock = threading.Lock()
    LogConfig = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'standard': {
                'format': '[%(asctime)s.%(msecs)03d] [%(name)s] [%(levelname)s] [%(filename)s:%(lineno)s] %(message)s',
                'datefmt': '%Y-%m-%d %H:%M:%S'
            }
        },
        'filters': {
        },
        'handlers': {
        },
        'loggers': {
        }
    }

    @classmethod
    def _get_log_level_from_env(cls, env_name, default_level=logging.DEBUG):
        log_level = str(os.environ.get(env_name)).lower()

        if log_level == "off":
            return logging.CRITICAL
        elif log_level == "critical":
            return logging.CRITICAL
        elif log_level == "err":
            return logging.ERROR
        elif log_level == "warn":
            return logging.WARNING
        elif log_level == "info":
            return logging.INFO
        elif log_level == "debug":
            return logging.DEBUG
        elif log_level == "trace":
            return logging.NOTSET
        else:
            return default_level

    @classmethod
    def _get_log_path_from_env(cls, path):
        log_path = get_log_given_path(path)
        if not os.path.exists(log_path):
            os.makedirs(log_path)
        if not os.path.isdir(log_path):
            raise TypeError("The log path '{}' is not a directory.".format(log_path))
        return log_path

    @classmethod
    def get_logger(cls, name, path, async_logging=True):
        with cls.LogConfigLock:
            cls.LogConfig['handlers'][f'file_{name}'] = copy.deepcopy(cls.LogHandlerSample['file'])
            cls.LogConfig['handlers'][f'console_{name}'] = copy.deepcopy(cls.LogHandlerSample['console'])

            if str(os.environ.get('USE_TIMED_LOG')).lower() == 'false':
                cls.LogConfig['handlers'][f'file_{name}']['class'] = 'logging.handlers.RotatingFileHandler'
                cls.LogConfig['handlers'][f'file_{name}']['maxBytes'] = get_digit_from_env('FILE_MAX_BYTES', 1073741824)  # 1GB = 1024 * 1024 * 1024
            else:
                cls.LogConfig['handlers'][f'file_{name}']['class'] = 'logging.handlers.TimedRotatingFileHandler'
                cls.LogConfig['handlers'][f'file_{name}']['when'] = 'MIDNIGHT'
                cls.LogConfig['handlers'][f'file_{name}']['interval'] = 1

            filename = name + time.strftime("%Y%m%d%H%M%S", time.localtime())
            cls.LogConfig['handlers'][f'file_{name}']['filename'] = os.path.join(cls._get_log_path_from_env(path), f'{filename}.log')
            cls.LogConfig['handlers'][f'console_{name}']['level'] = cls._get_log_level_from_env('CONSOLE_LOG_LEVEL', logging.ERROR)
            cls.LogConfig['handlers'][f'file_{name}']['level'] = cls._get_log_level_from_env('FILE_LOG_LEVEL', logging.INFO)
            cls.LogConfig['handlers'][f'file_{name}']['backupCount'] = get_digit_from_env('FILE_BACKUP_COUNT', 21)

            cls.LogConfig['loggers'][name] = {}
            cls.LogConfig['loggers'][name]['handlers'] = [f'file_{name}', f'console_{name}']
            cls.LogConfig['loggers'][name]['level'] = 'DEBUG'
            log_config = copy.deepcopy(cls.LogConfig)

        logging.config.dictConfig(log_config)
        if async_logging:
            return cls.make_handlers_async(name)
        else:
            return logging.getLogger(name)

    @staticmethod
    def _select_and_disable_console_handler(hdlrs):
        for hdlr in hdlrs:
            if hdlr.name.startswith('console_'):
                hdlr.setStream(open('/dev/null', 'w'))
            if isinstance(hdlr, QueueHandler):
                Logger._select_and_disable_console_handler(hdlr.listener.handlers)

    @classmethod
    def redirect_console_handler_to_DEVNULL(cls):
        # root_logger = logging.getLogger()
        with cls.LogConfigLock:
            loggers = [one for one in cls.LogConfig['loggers'].keys() if one != '']

        for l_name in loggers:
            logger = logging.getLogger(l_name)
            Logger._select_and_disable_console_handler(logger.handlers)

    @classmethod
    def make_handlers_async(cls, l_name: str):
        ''' wrap the logger's handlers with a queued handler '''
        # root_logger = logging.getLogger()
        logger = logging.getLogger(l_name)
        assert not any([isinstance(h, QueueHandler) for h in logger.handlers]), 'exists handlers which have already been made async'
        q = queue.Queue()
        qhandler = QueueHandler(q)
        qhandler.listener = QueueListener(q, *logger.handlers, respect_handler_level=True)
        qhandler.listener.start()
        logger.handlers = [qhandler]
        return logger

    @classmethod
    def stop(cls):
        cls.stop_async_handlers()

    @classmethod
    def stop_async_handlers(cls):
        ''' stop queued handler and flush all pending logs '''
        # root_logger = logging.getLogger()
        with cls.LogConfigLock:
            loggers = [one for one in cls.LogConfig['loggers'].keys() if one != '']

        for l_name in loggers:
            logger = logging.getLogger(l_name)
            for hdlr in logger.handlers:
                if isinstance(hdlr, QueueHandler) and hdlr.listener is not None:
                    hdlr.listener.stop()
                    hdlr.listener = None



import math
def decode_algo_type(order_type:str):
    algo_type = -1
    mapping = {
        'kf_twap_plus': 27,
        'kf_vwap_plus': 28,
        'smart_kf' : 29, #T0策略，smarT_I,该策略在“smarT策略交易”菜单中“策略类型”展示为smarT_卡方
        'smart_yr' : 30
    }
    algo_type = mapping.get(order_type, -1)
    return algo_type

def date_to_str(date):
    year = int(date / 10000)
    month = int((date - 10000 * year) / 100)
    day = date - 10000 * year - 100 * month
    str_date = f'{year:04d}-{month:02d}-{day:02d} '
    return str_date

def decode_exchange_id(str_exchange):
    exchange = 0
    if str_exchange == 'SH':
        exchange = 101        #EXCHANGE_SSE 
    elif str_exchange == 'SZ':
        exchange = 102        #EXCHANGE_SZE 
    return exchange

def encode_exchange_id(exchange):
    if exchange.startswith('6'):
        str_exchange = '.SH'
    elif exchange.startswith('5'):
        str_exchange = '.SH'
    elif exchange.startswith('2'):
        str_exchange = '.SH'
    elif exchange.startswith('0'):
        str_exchange = '.SZ'
    else:
        str_exchange = '.SZ'

    return str_exchange

def transfer_order_side(side):
    if side == 'long':
        return 1
    elif side == 'short':
        return 2
    else:
        return -1
    
def side_to_target_type(side):
    target_type = ''
    if side in [1,5]:
        target_type = 'buy'
    elif side == 2:
        target_type = 'sell'
    return target_type

def decode_mquant_status(status):
    new_status = ''
    if status == 0:
        new_status = 'active'
    elif status == 4:
        new_status = 'filled'
    elif status == 2:
        new_status = 'canceled'
    return new_status

def decode_symbol_type(symbol_type):
    if symbol_type == 0:
        return (1000000, 10000000)
    elif symbol_type == 1:
        return (500000, 2000000)
    elif symbol_type == 2:
        return (300000, 3000000)
    elif symbol_type == 3:
        return (100000, 1000000)

def check_split(volume, price, symbol_type):
    """_summary_ 判断是否需要拆单,返回volume_list

    Args:
        volume (int): 下单量
        price (float): last_price,估计价格
        symbol_type (int):0-普通沪深主板， 1-风险警示股票(ST), 2-创业板， 3-科创板)
    """
    volume_list = []
    max_price = 1.1 * price
    max_volume, max_amt = decode_symbol_type(symbol_type)
    if volume <= max_volume and volume <= max_amt / max_price:
        return [volume]
    else:
        _split_volume = int(min(max_volume, max_amt/max_price))
        split_volume = _split_volume // 1000 * 1000
        while volume > split_volume:
            volume_list.append(volume)
            volume -= split_volume
        volume_list.append(volume)
        return volume_list



# encoding: utf-8
import sys
import json
import argparse
import  datetime
import socket                             #导入socket模块
import signal
import threading
import time
import csv
from datetime import date,datetime, timezone, timedelta
from ..mongo_client_gateway import MongoClientTradeGateway
from .constants import (GATEWAY_NAME, FILES)
from .utils import (decode_algo_type, date_to_str, decode_exchange_id, transfer_order_side, decode_mquant_status, side_to_target_type, check_split, decode_exchange_id, encode_exchange_id)
from .string_buffer import TcpStringBuffer

class MquantServer(MongoClientTradeGateway):
    def __init__(self, config_filename, end_time):
        MongoClientTradeGateway.__init__(self, config_filename, end_time, GATEWAY_NAME)
        self.tcp_socket_init = False
        self.tcp_socket = None
        self.conn = None
        self.load_tg_setting(config_filename)
        self.insert_orders = {}
        self.oid_to_ref = {}
        self.oid_to_traded_vol = {}
        self.oid_to_traded_amt = {}
        self.mid_to_req = {}
        self.oid_to_req = {}
        self.oid_to_mid = {}
        self.oid_to_acc = {}
        self.db_id_to_oid = {}
        self.ref_to_oid = {}
        self.oid_to_local_ids = {}
        self.tcp_string_buffer = TcpStringBuffer()
        self.tcp_msg_lock = threading.Lock()
        self.order_msg_lock = threading.Lock()
        self.my_today = datetime.strftime(datetime.now(), "%Y-%m-%d")


    # 创建TCP连接
    def create_tcp_client(self):
        if self.tcp_socket_init is False:
            self.tcp_socket = socket.socket()                        #创建套接字
            self.tcp_socket.connect((self.tcp_host,self.tcp_port))   #主动初始化TCP服务器连接
            self.tcp_socket_init = True

    #发送TCP数据
    @MongoClientTradeGateway.error_handler
    def tcp_send(self, data):
        if self.tcp_socket_init is True:
            self.tcp_string_buffer.push(data)

    @MongoClientTradeGateway.error_handler
    def tcp_recv(self):
        while not self.is_stopped:
            if self.tcp_socket_init is True:
                try:
                    #接受对方发送过来的数据，最大接受4096字节
                    recv_data = self.tcp_socket.recv(1024).decode()          #接收数据
                    self.tcp_string_buffer.receive(recv_data)
                    self.logger.info(f'recv msg: {recv_data}')
                except json.JSONDecodeError as e:
                    self.logger.error(f'recv failed {recv_data} error:{e}')
                except ConnectionAbortedError as e:
                    self.logger.error(f'connection broken error: {e}')
    @MongoClientTradeGateway.error_handler
    def handle_msg(self):
        while not self.is_stopped:
            if not self.tcp_string_buffer.is_recv_str_empty():
                msg_list = self.tcp_string_buffer.get_recv()
                for msg in msg_list:
                    self.on_message(msg)
            time.sleep(1)
    @MongoClientTradeGateway.error_handler
    def handle_send_msg(self):
        while not self.is_stopped:
            if not self.tcp_string_buffer.is_send_str_empty():
                send_data = self.tcp_string_buffer.get_send()
                self.tcp_socket.sendall(send_data.encode())
    # tcp_socket.close()    #关闭套接字
    def load_tg_setting(self, config_filename):
        f = open(config_filename, encoding='gbk')
        setting = json.load(f)
        self.recv_msg_path = setting['recv_msg_path']
        self.recv_msg_path = self.recv_msg_path.replace('/', '//')
        self.tcp_host = setting['tcp_host']
        self.tcp_port = setting['tcp_port']
        self.account_type = {}
        for acc in self.accounts_run:
            self.account_type[acc] = setting['accounts'][acc]['account_type']

    @MongoClientTradeGateway.error_handler
    def monitor_algo_limit_order_insert(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                target_account_name = self.target_account_names[acc]
                buy_query = {'accountName': target_account_name}
                targets_collection = self.order_info_db[acc]['limit_stop']
                targets = targets_collection.find(buy_query)
                if targets.count() == 0:
                    self.logger.warning(f'[monitor_algo_order_insert] no_buy_target (acc){acc}')
                    continue
                for target in targets:
                    if target['_id'] not in self.order_db_ids:
                        self.order_db_ids.append(target['_id'])
                        self.req_limit_order_insert(target)
                    else:
                        self.logger.warning(f'[monitor_algo_order_insert] _id_existed (_id){target["_id"]}')
    @MongoClientTradeGateway.error_handler
    def req_limit_order_insert(self, obj):
        target_account_name = obj['accountName']
        acc = self.target_account_names_to_acc[target_account_name]
        ticker = str(obj['ticker']) + encode_exchange_id(obj['ticker'])
        account_id = self.account_id[acc]
        mid = str(obj['mid'])
        vol = int(obj['volume'])
        symbol_type = 0
        price = 0.0
        if 'price' in obj:
            price = float(obj['price'])
        entrust_type = 9
        if not obj.get('financingBuy') is None :
            financial_buy_s = obj["financingBuy"]
            if financial_buy_s == True:
                entrust_type = 6
            else:
                entrust_type = 9
        account_type = self.account_type[acc]
        volume_list = check_split(vol, price, symbol_type)
        for order_vol in volume_list:
            oid = str(self.gen_order_id())
            side = obj['direction']

            msg_type = 'req_limit_order'
            output_dict = {
                'msg_type' : msg_type,
                'oid' : oid,
                'price' : price,
                'entrust_type': entrust_type,
                "account_type": account_type,
                'fund_account' : account_id,
                'side' : str(side),
                'ticker': ticker,
                'volume' : order_vol
            }
            self.tcp_send(output_dict)
            db_id = obj['_id']
            order_dict = {
                'oid' : oid,
                'mid' : mid,
                'db_id' : db_id,
                'ticker' : ticker,
                'side' : side,
                'volume' : order_vol,
                'account_name' : target_account_name,
                'order_type' : 'limit',
                'start_time' : '0',
                'end_time' : '0' #凑数
            }

            self.logger.info(f'[req_plain_order_insert] insert_success (oid){oid} (mid){mid} (param){output_dict}')
        local_id_list = []
        with self.order_msg_lock:
            self.mid_to_req[mid] = order_dict
            self.oid_to_req[oid] = order_dict
            self.oid_to_mid[oid] = mid
            self.db_id_to_oid[db_id] = oid
            self.oid_to_acc[oid] = acc
            self.oid_to_traded_vol[oid] = 0
            self.oid_to_traded_amt[oid] = 0
            self.oid_to_local_ids[oid] = local_id_list
        target_collection =  self.order_info_db[acc]['limit_stop']
        query = {
                    '_id' : db_id
                }
        delete_res = target_collection.delete_one(query)
        self.logger.info(f'[on_req_order] used (target){obj}')

    @MongoClientTradeGateway.error_handler
    def monitor_algo_order_insert(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                target_account_name = self.target_account_names[acc]
                buy_query = {'accountName': target_account_name}
                targets_collection = self.order_info_db[acc]['target']
                targets = targets_collection.find(buy_query)
                if targets.count() == 0:
                    self.logger.warning(f'[monitor_algo_order_insert] no_buy_target (acc){acc}')
                    continue
                for target in targets:
                    if target['_id'] not in self.order_db_ids:
                        self.order_db_ids.append(target['_id'])
                        self.req_order_insert(target, True)
                    else:
                        self.logger.warning(f'[monitor_algo_order_insert] _id_existed (_id){target["_id"]}')
            for acc in self.accounts_run:
                target_account_name = self.target_account_names[acc]
                buy_query = {'accountName': target_account_name}
                targets_collection = self.order_info_db[acc]['sell_target']
                targets = targets_collection.find(buy_query)
                if targets.count() == 0:
                    self.logger.warning(f'[monitor_algo_order_insert] no_sell_target (acc){acc}')
                    continue
                for target in targets:
                    if target['_id'] not in self.order_db_ids:
                        self.order_db_ids.append(target['_id'])
                        self.req_order_insert(target, False)
                    else:
                        self.logger.warning(f'[monitor_algo_order_insert] _id_existed (_id){target["_id"]}')

            time.sleep(self.scan_interval)

    @MongoClientTradeGateway.error_handler
    def req_plain_order_insert(self, obj):
        target_account_name = obj['accountName']
        acc = self.target_account_names_to_acc[target_account_name]
        account_id = self.account_id[acc]
        mid = str(obj['mid'])
        vol = int(obj['volume'])
        ticker = str(obj['ticker']) + encode_exchange_id(obj['ticker'])
        account_type = self.account_type[acc]
        symbol_type = 0
        price = 0.0
        if 'price' in obj:
            price = float(obj['price'])
        volume_list = check_split(vol, price, symbol_type)
        for order_vol in volume_list:
            oid = str(self.gen_order_id())
            side = obj['direction']

            msg_type = 'req_plain_order'
            output_dict = {
                'msg_type' : msg_type,
                'oid' : oid,
                'price' : price,
                'fund_account' : account_id,
                'side' : str(side),
                'account_type': account_type,
                'ticker': ticker,
                'volume' : order_vol,

            }
            self.tcp_send(output_dict)
            db_id = obj['_id']
            order_dict = {
                'oid' : oid,
                'mid' : mid,
                'db_id' : db_id,
                'ticker' : ticker,
                'side' : side,
                'volume' : order_vol,
                'account_name' : target_account_name,
                'order_type' : str(obj['order_type']),
                'start_time' : str(obj['start_time']),
                'end_time' : str(obj['end_time'])
            }

            self.logger.info(f'[req_plain_order_insert] insert_success (oid){oid} (mid){mid} (param){output_dict}')
        local_id_list = []
        with self.order_msg_lock:
            self.mid_to_req[mid] = order_dict
            self.oid_to_req[oid] = order_dict
            self.oid_to_mid[oid] = mid
            self.db_id_to_oid[db_id] = oid
            self.oid_to_acc[oid] = acc
            self.oid_to_traded_vol[oid] = 0
            self.oid_to_traded_amt[oid] = 0
            self.oid_to_local_ids[oid] = local_id_list
        target_collection =  self.order_info_db[acc]['limit_stop']
        query = {
                    '_id' : db_id
                }
        delete_res = target_collection.delete_one(query)
        self.logger.info(f'[on_req_order] used (target){obj}')

    @MongoClientTradeGateway.error_handler
    def req_batch_order_insert(self, obj, side):
        msg_type = 'req_batch_order_insert'
        target_account_name = obj['accountName']
        acc = self.target_account_names_to_acc[target_account_name]
        account_id = self.account_id[acc]
        mid = str(obj['mid'])
        batch_oid = str(self.gen_order_id())
        algo_type = decode_algo_type(obj['order_type'])
        str_date = date_to_str(int(self.date))
        start_time  = str_date + str(obj['start_time'])
        end_time = str_date + str(obj['end_time'])
        limit_action = 1  #涨跌停后交易
        if not obj.get('LimAction') is None :
            if obj.get('LimAction'):
                limit_action = 0
        after_action = 0    #时间结束后不交易
        if not obj.get('AftAction') is None :
            if obj.get('AftAction'):
                after_action = 1
        up_limit = -1 #涨幅限制
        if not obj.get('up_limit') is None:
            up_limit = obj['up_limit']
        down_limit = -1 #涨幅限制
        if not obj.get('down_limit') is None:
            down_limit = obj['down_limit']
        limit = 0 #限价
        if not obj.get('limit') is None:
            limit = obj['limit']
        expire_date =  datetime.strftime(datetime.today(), '%Y-%m-%d')
        if not obj.get('expire_date') is None:
            expire_date = obj['expire_date']
        occupy_amount = 0.0
        if not obj.get('occupy_amount') is None:
            occupy_amount = obj['occupy_amount']
        param_list = []
        db_id = obj['_id']
        for param in obj['param_list']:
            oid = str(self.gen_order_id())
            stk_id = param['stk_id']
            volume = param['volume']
            algo_order_info = {
                'oid' : oid,
                'stk_id' : stk_id,
                'volume' : volume
            }
            param_list.append(algo_order_info)
            order_dict = {
                'oid' : oid,
                'mid' : mid,
                'db_id' : db_id,
                'ticker' : param['stk_id'],
                'side' : side,
                'volume' : volume,
                'account_name' : target_account_name,
                'order_type' : str(obj['order_type']),
                'start_time' : str(obj['start_time']),
                'end_time' : str(obj['end_time'])
            }
            local_id_list = []
            with self.order_msg_lock:
                #self.mid_to_req[mid] = order_dict
                self.oid_to_req[oid] = order_dict
                self.oid_to_mid[oid] = mid
                self.db_id_to_oid[db_id] = oid
                self.oid_to_acc[oid] = acc
                self.oid_to_traded_vol[oid] = 0
                self.oid_to_traded_amt[oid] = 0
                self.oid_to_local_ids[oid] = local_id_list
        output_dict = {
                'msg_type' : msg_type,
                'oid' : batch_oid,
                'algo_type' : algo_type,
                'fund_account' : account_id,
                'side' : str(side),
                'start_time' : start_time,
                'end_time' : end_time,
                'up_limit': up_limit,
                'down_limit': down_limit,
                'param_list' : param_list,
                'limit' : limit,
                'limit_action': limit_action,
                'after_action': after_action,
                'expire_date': expire_date,
                'occupy_amount': occupy_amount
            }
        self.tcp_send(output_dict)
        self.logger.info(f'[req_batch_order_insert] insert_success (mid){mid} (param){output_dict}')
        target_collection =  self.order_info_db[acc]['target']
        query = {
                    '_id' : db_id
                }
        delete_res = target_collection.delete_one(query)
        self.logger.info(f'[on_req_order_insert] used (target){obj}')

    @MongoClientTradeGateway.error_handler
    def req_order_insert(self, obj, IsBuy):
        target_account_name = obj['accountName']
        acc = self.target_account_names_to_acc[target_account_name]
        side = 1
        if not IsBuy:
            side = 2
        order_type = obj['executionPlan']['order_type']
        if order_type == 'smart_yr':
            self.req_batch_order_insert(obj, side)
            return
        entrust_type = 0
        if not obj.get('financingBuy') is None :
            financial_buy_s = obj["financingBuy"]
            if financial_buy_s == True:
                entrust_type = 6
            else:
                entrust_type = 9
        account_id = self.account_id[acc]
        account_type = self.account_type[acc]
        symbol_type = 0
        #price = float(obj['last_price'])
        mid = str(obj['mid'])
        vol = int(obj['volume'])
        algo_type = decode_algo_type(obj['executionPlan']['order_type'])
        str_date = date_to_str(int(self.date))
        start_time  = str_date + str(obj['executionPlan']['start_time'])
        end_time = str_date + str(obj['executionPlan']['end_time'])
        ticker = str(obj['ticker']) + encode_exchange_id(obj['ticker'])
        limit_action = 1  #涨跌停后交易
        if not obj.get('LimAction') is None :
            if obj.get('LimAction'):
                limit_action = 0
        after_action = 0    #时间结束后不交易
        if not obj.get('AftAction') is None :
            if obj.get('AftAction'):
                after_action = 1
        up_limit = -1 #涨幅限制
        if not obj.get('up_limit') is None:
            up_limit = obj['up_limit']
        down_limit = -1 #涨幅限制
        if not obj.get('down_limit') is None:
            down_limit = obj['down_limit']
        expire_date =  datetime.strftime(datetime.today(), '%Y-%m-%d')
        if not obj.get('expire_date') is None:
            expire_date = obj['expire_date']
        occupy_amount = 0.0
        if not obj.get('occupy_amount') is None:
            occupy_amount = obj['occupy_amount']
        price = 0.0
        if not obj.get('price') is None:
            price = float(obj['price'])
        #volume_list = check_split(vol, price, symbol_type)
        
        oid = str(self.gen_order_id())
        msg_type = 'req_order_insert'
        output_dict = {
            'msg_type' : msg_type,
            'oid' : oid,
            'algo_type' : algo_type,
            'fund_account' : account_id,
            'price' : price,
            'side' : str(side),
            'ticker': ticker,
            'volume' : vol,
            'start_time' : start_time,
            'end_time' : end_time,
            'up_limit': up_limit,
            'down_limit': down_limit,
            'limit' : price,
            'limit_action': limit_action,
            'after_action': after_action,
            'expire_date': expire_date,
            'occupy_amount': occupy_amount,
            'entrust_type': entrust_type,
            'account_type': account_type
        }
        self.tcp_send(output_dict)
        db_id = obj['_id']
        order_dict = {
            'oid' : oid,
            'mid' : mid,
            'db_id' : db_id,
            'ticker' : ticker,
            'side' : side,
            'volume' : vol,
            'account_name' : target_account_name,
            'order_type' : order_type,
            'start_time' : start_time,
            'end_time' : end_time
        }
        local_id_list = []
        with self.order_msg_lock:
            self.mid_to_req[mid] = order_dict
            self.oid_to_req[oid] = order_dict
            self.oid_to_mid[oid] = mid
            self.db_id_to_oid[db_id] = oid
            self.oid_to_acc[oid] = acc
            self.oid_to_traded_vol[oid] = 0
            self.oid_to_traded_amt[oid] = 0
            self.oid_to_local_ids[oid] = local_id_list
        self.logger.info(f'[req_order_insert] insert_success (oid){oid} (mid){mid} (param){output_dict}')
        target_collection =  self.order_info_db[acc]['target']
        query = {
                    '_id' : db_id
                }
        delete_res = target_collection.delete_one(query)
        self.logger.info(f'[on_req_order_insert] used (target){obj}')

    @MongoClientTradeGateway.error_handler
    def on_message(self, msg):
        """ _summary_ : 根据信息内容，判断 on_order_msg/on_trade_msg/.../ 所有回报都在这个函数
            Args:
                msg (_type_): dict
        """
        if not 'msg_type' in msg:
            self.logger.error(f'msg_type not exist msg:{msg}')
        else:
            msg_type = msg['msg_type']
            if msg_type == 'on_rsp_order_insert':
                self.on_rsp_order_insert(msg)
            elif msg_type == 'on_rsp_plain_order_insert':
                self.on_rsp_plain_order_insert(msg)
            elif msg_type == 'on_rtn_trade':
                self.on_trade_msg(msg)
            elif msg_type == "on_rtn_order":
                if msg['status'] in [0 ,2, 4]:
                    self.update_order(msg)
                elif msg['status'] == 3:
                    self.logger.warning(f'order rejected: msg{msg}')
            elif msg_type == 'on_position':
                self.on_rtn_position(msg)
            elif msg_type == 'on_fund':
                self.on_rtn_fund(msg)
            else:
                self.logger.error(f'can not parse msg: {msg}')

    @MongoClientTradeGateway.error_handler
    def on_rsp_order_insert(self, msg):
        is_success = msg['status']
        err_info = msg['err_info']
        oid = msg['oid']
        inst_id = msg['inst_id'] #实例ID
        if not is_success:
            self.logger.warning(f'[on_rsp_order_insert] req insert failed oid : {oid}, error_info:{err_info}')

        self.oid_to_ref[oid] = inst_id
        self.ref_to_oid[inst_id] = oid
        self.logger.info(f'[on_rsp_order_insert] insert success oid:{oid}')
        acc = self.oid_to_acc[oid]
        mid = self.oid_to_mid[oid]
        order_dict = self.oid_to_req[oid]
        cached_order_collection = self.order_info_db[acc]['cached_order']
        local_id_list = []
        db_msg = {
                    'oid' : oid,
                    'mid' : mid,
                    'mudan_id' : inst_id,
                    "insert_time": datetime.utcnow(),
                    'traded_vol' : 0,
                    'traded_amt' : 0,
                    'local_ids' : local_id_list,
                    'order_msg': order_dict
        }
        query = {'oid': oid}
        res = cached_order_collection.replace_one(query, db_msg, True)
        self.logger.info(f'[on_rsp_order_insert] update_cached_order_info (res){res} (msg){db_msg}')

    @MongoClientTradeGateway.error_handler
    def on_rsp_plain_order_insert(self, msg):
        oid = msg['oid']
        order_id = msg['order_id'] #实例ID

        self.oid_to_ref[oid] = order_id
        self.ref_to_oid[order_id] = oid
        self.logger.info(f'[on_rsp_order_insert] insert success oid:{oid}')
        acc = self.oid_to_acc[oid]
        mid = self.oid_to_mid[oid]
        order_dict = self.oid_to_req[oid]
        cached_order_collection = self.order_info_db[acc]['cached_order']
        local_id_list = []
        db_msg = {
                    'oid' : oid,
                    'mid' : mid,
                    'mudan_id' : order_id,
                    "insert_time": datetime.utcnow(),
                    'traded_vol' : 0,
                    'traded_amt' : 0,
                    'local_ids' : local_id_list,
                    'order_msg': order_dict
        }
        query = {'oid': oid}
        res = cached_order_collection.replace_one(query, db_msg, True)
        self.logger.info(f'[on_rsp_order_insert] update_cached_order_info (res){res} (msg){db_msg}')

    @MongoClientTradeGateway.error_handler
    def monitor_trade_update(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                trade_filename = self.recv_msg_path + '//' + self.my_today + '//' + FILES.TRADES + '_' + trade_acc + '.csv'
                with open(trade_filename, 'r', encoding='utf-8') as trade_file:
                    csv_dict_reader = csv.DictReader(trade_file)
                    for row in csv_dict_reader:
                        self.on_trade_msg(row)
            time.sleep(self.trade_scan_interval)

    @MongoClientTradeGateway.error_handler
    def on_trade_msg(self, msg):
        if msg['amount'] == 0 or msg['price'] == 0:
            self.logger.warning(f'[on_trade_msg] nothing_traded (record){msg}')
            return
        trade_acc = msg['fund_account']
        mudan_id = msg['algo_inst_id']
        if mudan_id is None or len(mudan_id) == 0:
            if 'order_id' not in msg or len(msg['order_id']) == 0:
                self.logger.warning(f"[on_trade_msg] can not get mudan_id, msg:{msg}")
                return
            mudan_id = msg['order_id']
        oid = ''
        acc = ''
        mid = ''
        order_dict = {}
        with self.order_msg_lock:
            if mudan_id not in self.ref_to_oid or oid not in self.oid_to_acc:
                acc = self.account_id_to_acc[trade_acc]
                order_info_collection = self.order_info_db[acc]['cached_order']
                query = {'mudan_id' : mudan_id}
                order_info_target = order_info_collection.find_one(query)
                if not order_info_target is None:
                    order_dict = order_info_target['order_msg']
                    mid = order_info_target['mid']
                    oid = order_info_target['oid']
                    self.oid_to_local_ids[oid] = order_info_target['local_ids']
                    self.oid_to_traded_vol[oid] = 0
                    self.oid_to_traded_amt[oid] = 0
                    self.oid_to_mid[oid] = mid
                    self.oid_to_req[oid] = order_dict
                    self.oid_to_ref[oid] = mudan_id
                    self.ref_to_oid[mudan_id] = oid
                else:
                    self.logger.warning(f"[on_trade_msg] can not get oid, msg:{msg}, mudan_id:{mudan_id}")
                    return
            else:
                oid = self.ref_to_oid[mudan_id]
                acc = self.oid_to_acc[oid]
                order_dict = self.oid_to_req[oid]
                mid = self.oid_to_mid[oid]
        acc = self.account_id_to_acc[trade_acc]
        order_info_collection = self.order_info_db[acc]['cached_order']
        query = {'oid' : oid}
        order_info_target = order_info_collection.find_one(query)

        _id = str(msg['trade_id'])
        if _id in self.oid_to_local_ids[oid]:
            self.logger.warning(f'[on_trade_msg] duplicate_trade_msg (record){msg}')
            return
        else:
            self.oid_to_local_ids[oid].append(_id)#外部委托编号
        ticker = str(msg['symbol']).split('.')[0]
        exchange = decode_exchange_id(str(msg['symbol']).split('.')[1])
        traded_vol = int(msg['amount'])
        traded_price = float(msg['price'])
        trade_amt = float(msg['business_balance'])
        order_type = msg['real_type']
        target_type = int(msg['side'])
        if target_type in [1, 3]:
            self.oid_to_traded_amt[oid] += trade_amt
            self.oid_to_traded_vol[oid] += traded_vol
        elif target_type == 2:
            self.oid_to_traded_amt[oid] -= trade_amt
            self.oid_to_traded_vol[oid] -= traded_vol
        dbTime = datetime.now(timezone.utc)
        str_trade_time = msg['time']
        trade_time = datetime.strptime(str_trade_time, '%Y-%m-%d %H:%M:%S')
        utc_trade_time = trade_time - timedelta(hours=8)
        target_account_name = order_dict['account_name']
        acc = self.target_account_names_to_acc[target_account_name]
        log_account_name = order_dict['account_name'].split('@')[1]
        tg_name =  order_dict['account_name'].split('@')[0]

        db_msg = {
            'trade_ref': _id,
            'oid': int(oid),
            'tg_name': tg_name,
            'ticker': ticker,
            "exchange": exchange,
            'traded_vol': traded_vol,
            'traded_price': traded_price,
            'order_type': order_type, #算法类型
            'side': target_type,
            'entrust_vol': 0,
            'entrust_price': 0,
            'dbTime': dbTime,
            'mid': mid,  # 对应订单中的 mid
            "commission" : 0,
            'trade_time': utc_trade_time,  # 具体交易时间
            'accountName':  log_account_name
        }
        trade_collection = self.tradelog_db[acc]['tg_trade']
        replace_trade_query = { 'trade_ref': _id, 'oid' : int(oid), 'mid': mid, 'accountName': log_account_name}
        db_res = trade_collection.replace_one(replace_trade_query, db_msg, True)
        self.logger.info(
            f'[rtn_trade] (db_res){db_res} (db_msg){db_msg} (traded_vol){traded_vol} (traded_price){traded_price}')
        #self.update_position(traded_vol, order_dict, mid, oid, target_type, trade_amt,exchange)

        cached_order_collection = self.order_info_db[acc]['cached_order']
        cached_order_msg = {
                    'local_ids' : self.oid_to_local_ids[oid],
                }
        update_cached_order_msg = {'$set': cached_order_msg}
        query = {'oid': oid}
        res = cached_order_collection.update_one(query, update_cached_order_msg)
        self.logger.info(f'[on_trade_msg] update_cached_order_info (res){res} (msg){update_cached_order_msg}')

    def monitor_order_update(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                order_filename = self.recv_msg_path + '//' + self.my_today + '//' + FILES.ORDERS + '_' + trade_acc + '.csv'
                with open(order_filename, 'r', encoding='utf-8') as order_file:
                    csv_dict_reader = csv.DictReader(order_file)
                    for row in csv_dict_reader:
                        status = int(row['status'])
                        if status in [2 ,4]:
                            self.update_order(row)
            time.sleep(self.order_scan_interval)

    @MongoClientTradeGateway.error_handler
    def update_order(self, msg):
        oid = ''
        acc = ''
        mid = ''
        order_dict = {}
        mudan_id = msg['algo_inst_id']
        if mudan_id is None or len(mudan_id) == 0:
            if 'order_id' not in msg or len(msg['order_id']) == 0:
                self.logger.warning(f"[update_order] can not get mudan_id, msg:{msg}")
                return
            mudan_id = msg['order_id']
        trade_acc = msg['fund_account']
        acc = self.account_id_to_acc[trade_acc]
        order_info_collection = self.order_info_db[acc]['cached_order']
        with self.order_msg_lock:
            if  mudan_id not in self.ref_to_oid or oid not in self.oid_to_acc: #shut_down or other problem
                if trade_acc not in self.account_id_to_acc:
                    self.logger.error(f'[update_order] can not_parse_trade_acc {trade_acc}')
                    return
                query = {'mudan_id' : mudan_id}
                order_info_target = order_info_collection.find_one(query)
                if not order_info_target is None:
                    order_dict = order_info_target['order_msg']
                    oid = order_info_target['oid']
                    mid = order_info_target['mid']
                    self.oid_to_local_ids[oid] = order_info_target['local_ids']
                    self.oid_to_traded_amt[oid] = 0
                    self.oid_to_traded_vol[oid] = 0
                    self.oid_to_mid[oid] = mid
                    self.oid_to_req[oid] = order_dict
                    self.oid_to_ref[oid] = mudan_id
                    self.ref_to_oid[mudan_id] = oid
                else:
                    self.logger.warning(f'[update_order] can not_find_oid (mudan_id){mudan_id}')
                    return
            else:
                oid = self.ref_to_oid[mudan_id]
                acc = self.oid_to_acc[oid]
                order_dict = self.oid_to_req[oid]
                mid = self.oid_to_mid[oid]
        local_id = msg['order_id']
        if local_id in self.oid_to_local_ids[oid]:
            self.logger.warning(f'[update_order] duplicate_order_msg (record){msg}')
            return
        else:
            self.oid_to_local_ids[oid].append(local_id)
        ticker = msg['symbol'].split('.')[0]
        exchange = decode_exchange_id(str(msg['symbol'].split('.')[1]))
        volume = order_dict['volume']
        price = float(msg['price'])

        tg_name = order_dict['account_name'].split('@')[0]
        account_name = order_dict['account_name'].split('@')[1]
        order_type = order_dict['order_type']
        start_time = order_dict['start_time']
        end_time = order_dict['end_time']

        target_type = side_to_target_type(order_dict['side'])
        filled_vol = int(msg['filled'])
        status = decode_mquant_status(int(msg['status']))

        replace_collection = self.tradelog_db[acc]['order']
        query = {'mid':mid, '_id': int(oid)}
        replace_target = replace_collection.find_one(query)
        if not replace_target is None:
            utc_update_time = replace_target['dbTime']
            _filled_vol = filled_vol + replace_target['filled_vol']
            filled_vol = min(_filled_vol, volume)
            order_msg = {
                '_id': int(oid),
                'tg_name': tg_name,  # 对应 account_info._ids
                "exchange": exchange,
                'target_type': target_type,    # 'buy' | 'sell' | 'limit_sell'
                'volume': volume,  # 订单的volume
                'price': price,  # 实际成交均价
                'order_type': order_type,
                'ticker': ticker,
                'mid': mid,  # target中对应的 母单mid
                'accountName': account_name,
                'algo_args': {  # 具体的算法参数
                    'order_type': order_type,  # 此柜台不使用下单时要求的算法
                    'start_time': start_time,
                    'end_time': end_time
                },
                'status': status,  # 'active' | 'filled' | 'canceled'
                'filled_vol': filled_vol,  # 实际成交的 volume
                'dbTime': utc_update_time,
            }

            res = replace_collection.replace_one(query, order_msg, True)
            self.logger.info(f'[rtn_order] (res){res} (order_msg){order_msg}')
        else:
            str_update_time = str((msg['add_time']))
            update_time = datetime.strptime(str_update_time, '%Y-%m-%d %H:%M:%S')
            utc_update_time = update_time - timedelta(hours=8)
            order_msg = {
                '_id': int(oid),
                'tg_name': tg_name,  # 对应 account_info._id
                'target_type': target_type,    # 'buy' | 'sell' | 'limit_sell'
                "exchange": exchange,
                'volume': volume,  # 订单的volume
                'price': price,  # 实际成交均价
                'order_type': order_type,
                'ticker': ticker,
                'mid': mid,  # target中对应的 母单mid
                'accountName': account_name,  # 对应 account_info.account_name
                'algo_args': {  # 具体的算法参数
                    'order_type': order_type,
                    'start_time': start_time,
                    'end_time': end_time
                },
                'status': status,  # 'active' | 'filled' | 'canceled'
                'filled_vol': filled_vol,  # 实际成交的 volume
                'dbTime': utc_update_time,
            }

            res = replace_collection.replace_one(query, order_msg, True)
            self.logger.info(f'[rtn_order] (res){res} (order_msg){order_msg}')
        cached_order_collection = self.order_info_db[acc]['cached_order']
        cached_order_msg = {
                    'local_ids' : self.oid_to_local_ids[oid],
                }
        update_cached_order_msg = {'$set': cached_order_msg}
        query = {'oid': oid}
        res = cached_order_collection.update_one(query, update_cached_order_msg)
        self.logger.info(f'[update_order] update_cached_order_info (res){res} (msg){update_cached_order_msg}')

    @MongoClientTradeGateway.error_handler
    def monitor_position_update(self):
        while not self.is_stopped:
            print("[monitor_position_update]")
            for acc in self.accounts_run:
                pos_msg_list = []
                trade_acc = self.account_id[acc]
                tg_name = self.tgnames[acc]
                pos_collection = self.order_info_db[acc]['tg_equity_position']
                delete_filter = {
                    "tg_name" : tg_name
                }
                delete_res = pos_collection.delete_many(delete_filter)
                self.logger.info(f"delete {delete_res.deleted_count} documents in {tg_name}")
                pos_filename = self.recv_msg_path + '//' + self.my_today + '//' + FILES.POSITIONS + '_' + trade_acc + '.csv'
                try:
                    with open(pos_filename, 'r', encoding='utf-8') as pos_file:
                        csv_dict_reader = csv.DictReader(pos_file)
                        for row in csv_dict_reader:
                            pos_msg = self.on_rtn_position(row)
                            pos_msg_list.append(pos_msg)
                    if len(pos_msg_list) > 0:
                        try:
                            res = pos_collection.insert_many(pos_msg_list)
                        except Exception as e:
                            self.logger.error(f'insert error: {e}')
                            time.sleep(1)
                            continue
                        self.logger.info(f'[on_rtn_position] position_updated length:{len(res.inserted_ids)}')
                except Exception as e:
                    self.logger.error(f'[on_rtn_position] insert error: {e}')
                    time.sleep(1)
                    continue

            time.sleep(self.pos_interval)
    @MongoClientTradeGateway.error_handler
    def on_rtn_position(self, msg):

        trade_acc = msg['fund_account']
        if trade_acc not in self.account_id_to_acc:
            self.logger.warning(f'[on_rtn_position] can not_parse_trade_acc {trade_acc}')
            return
        acc = self.account_id_to_acc[trade_acc]
        account_name = self.log_account_names[acc]
        tg_name = self.tgnames[acc]
        ticker = str(msg['symbol']).split('.')[0]
        exchange = decode_exchange_id(str(msg['symbol']).split('.')[1])
        td_pos = float(msg['total_amount'])
        yd_pos = float(msg['closeable_amount'])
        market_value = float(msg['value'])
        holding_cost = float(msg['holding_cost'])
        pos_msg = {
            "account_name": account_name,
            'tg_name': tg_name,
            'ticker': ticker,
            "exchange": exchange,
            "direction": "long",
            'avail_pos': yd_pos, # 昨仓
            'total_pos': td_pos, # 今仓
            "type": "stock",
            'updated_at': datetime.utcnow()
        }
        return pos_msg

    @MongoClientTradeGateway.error_handler
    def monitor_fund_update(self):
        while not self.is_stopped:
            print("[monitor_fund_update]")
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                asset_filename = self.recv_msg_path + '//' + self.my_today + '//' + FILES.ASSETS + '_' + trade_acc + '.csv'
                with open(asset_filename, 'r', encoding='utf-8') as asset_file:
                    csv_dict_reader = csv.DictReader(asset_file)
                    for row in csv_dict_reader:
                        self.on_rtn_fund(row)
            time.sleep(self.acc_interval)
    def on_rtn_fund(self, msg):
        trade_acc = msg['fund_account']
        available_cash = float(msg['available_cash'])
        market_amt = float(msg['market_value'])
        acc = self.account_id_to_acc[trade_acc]
        tg_name = self.tgnames[acc]
        account_name = self.log_account_names[acc]
        asset_collection = self.order_info_db[acc]['EquityAccount']
        query = {'tg_name':tg_name, 'account_name': account_name}
        id_query = {'_id': tg_name}
        target = asset_collection.find_one(id_query)
        if target is None:
            asset_msg = {
                "_id" : tg_name,
                #"tg_name" : tg_name, # 对应之前的 TG 名称
                'accountName': account_name,  # 产品名称
                #"fund_account" : trade_acc,
                'avail_amt': available_cash,  # 可用资金（可现金买入的资金）
                'balance': available_cash + market_amt, # 净资产.  最好是由券商那边提供，通常普通账户是 可用资金(包含预扣费用) + 市值；信用账户是总资金 - 总负债
                'holding': market_amt, # 市值,
                'updated_at': datetime.utcnow()
            }
            res = asset_collection.insert_one(asset_msg)
            self.logger.info(f'[update_dbf_asset] (res){res} (order_msg){asset_msg}')
        else:
            up_asset_msg = {
                #"tg_name" : tg_name, # 对应之前的 TG 名称
                'accountName': account_name,  # 产品名称
                #"fund_account" : trade_acc,
                'avail_amt': available_cash,  # 可用资金（可现金买入的资金）
                'balance': available_cash + market_amt, # 净资产.  最好是由券商那边提供，通常普通账户是 可用资金(包含预扣费用) + 市值；信用账户是总资金 - 总负债
                'holding': market_amt, # 市值,
                'updated_at': datetime.utcnow()
            }
            res = asset_collection.update_one(id_query, {'$set': up_asset_msg}, True)
            self.logger.info(f'[update_dbf_asset] (res){res} (order_msg){up_asset_msg}')

    def monitor(self):
        ts = [
            threading.Thread(target=self.monitor_algo_order_insert),
            threading.Thread(target=self.tcp_recv),
            threading.Thread(target=self.handle_send_msg),
            threading.Thread(target=self.handle_msg),
            threading.Thread(target=self.monitor_fund_update),
            threading.Thread(target=self.monitor_position_update),
            threading.Thread(target=self.monitor_trade_update),
            threading.Thread(target=self.monitor_order_update),
            threading.Thread(target=self.monitor_algo_limit_order_insert),
            threading.Thread(target=self.date_change)
        ]
        for t in ts:
            t.setDaemon(True)
            t.start()
        #self.thread_pool.submit(self.monitor_dbf_pos)

    @MongoClientTradeGateway.error_handler
    def start(self):
        signal.signal(signal.SIGTERM, self.signal_handler)
        signal.signal(signal.SIGINT, self.signal_handler)
        _msg = f'[login] {GATEWAY_NAME}_server_start (time){datetime.now()}'
        self.send_to_user(logger=self.logger, url_list=self.url_list, msg=_msg)
        self.create_tcp_client()
        #self.update_holding_pos("pre_holding_pos")
        self.monitor()

    def close(self):
        self.tcp_socket_init = False
        self.tcp_socket.close()
        return super().close()

    def join(self):
        while self.is_stopped == False:
            time.sleep(0.01)
            if self.is_stopped:
                self.logger.info(
                    '[close] main thread is stopped,active_orders message will lose')
                self.close()

if __name__ == '__main__':
    sys.stdout.reconfigure(encoding='utf-8')
    description = 'mquant_server,get target from mongodb and serve mquant api'  
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('-e' , '--end_time', dest='end_time', default='15:30')
    _config_filename = 'C:/Users/Administrator/Desktop/mquant_batandconfig/mquant_server_config.json'
    parser.add_argument('-p', '--config_filepath', dest= 'config_filepath', default= _config_filename)

    args = parser.parse_args()
    print (f'(args){args}')

    td = MquantServer(args.config_filepath, args.end_time)
    td.start()

    td.join()



import sys
import json
from datetime import datetime
import traceback
import argparse

from ..mongo_client_gateway import MongoClientTradeGateway
from .constants import (FILES, GATEWAY_NAME)
from .utils import str2bool


class MquantTool(MongoClientTradeGateway):
    def __init__(self, config_filename, sync_pos,  end_time, clear_atx, gateway_name):
        MongoClientTradeGateway.__init__(
            self, config_filename, end_time, gateway_name)
        self.update_trade_date = self.date
        self.sync_pos = sync_pos
        self.clear_atx = clear_atx
        self.load_tool_setting(config_filename)

    def load_tool_setting(self, config_filename):
        try:
            #config_filename = os.path.join(config_path, 'atx_server_config.json')
            f = open(config_filename, encoding='gbk')
            setting = json.load(f)
            #self.recv_msg_dir = setting['recv_msg_path'].replace('/', '\\')
            self.sync_position_open = setting['sync_position_open']
            self.sync_position_close = setting['sync_position_close']

        except Exception as e:
            err = traceback.format_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            traceback.print_exception(
                exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
            print(f"load config failed! (exception){err}")
            exit(0)

    @MongoClientTradeGateway.error_handler
    def clear_atx_order(self):
        for acc in self.accounts_run:
            collection = self.order_info_db[acc]['atx_order']
            target_account_name = self.target_account_names[acc]
            targets_query = {'order_msg.accountName': target_account_name}
            targets = collection.find(targets_query)
            self.logger.info(f"now (target_account_name){target_account_name}")
            if targets.count() == 0:
                continue
            for order_tick in targets:
                local_ids = []
                instr_ids = []
                self.logger.info(f"goes in (order_tick){order_tick}")
                sid = order_tick['sid']
                query = {'sid': sid, 'order_msg.accountName': target_account_name}
                change = {
                    'local_ids': local_ids,
                    'instr_ids': instr_ids
                }
                new_data = {"$set": change}
                res = collection.update_many(
                    query, new_data, True)
                self.logger.info(
                    f"[update_atx_order] (res){res} (change){change}")

    def start(self):

        if self.sync_pos == True:
            self.date_change()
            _msg = "[gf_date_change] executed"
            self.send_to_user(logger=self.logger,
                              url_list=self.url_list, msg=_msg)
        if self.clear_atx == True:
            self.clear_atx_order()
            _msg = "[gf_clear_atx] executed"
            self.send_to_user(logger=self.logger,
                              url_list=self.url_list, msg=_msg)

    @MongoClientTradeGateway.error_handler
    def date_change(self):
        dt = datetime.now()
        self.logger.info(f"(dt){dt.hour}")
        if dt.hour <= self.sync_position_open and self.sync_pos == True:
            self.logger.info("[date_change] date_change_open")
            self.update_position_date_open()
        elif dt.hour < self.sync_position_close and dt.hour > self.sync_position_open:
            self.logger.info("[date_change] date_not_change")
        elif dt.hour >= self.sync_position_close and self.sync_pos == True:
            self.logger.info("[date_change] date_change_close")
            self.update_position_date_close()
        else:
            self.logger.info("[date_change] date_not_change")
            return


if __name__ == "__main__":

    description = "sync pos for emc_server"
    parser = argparse.ArgumentParser(description=description)

    _file_name = "C:\\Users\\Administrator\\Desktop\\emc_batandconfig\\emc_tool_setting.json"
    _date = datetime.now().strftime("%Y%m%d")
    parser.add_argument('-m', '--config_filename',
                        dest='config_filename', type=str, default=_file_name)
    parser.add_argument('-p', '--date_change',
                        dest='date_change', type=str2bool, default='T')
    parser.add_argument('-c', '--clear_atx', dest='clear_atx',
                        type=str2bool, default='F')
    end_time = "15:30"
    args = parser.parse_args()
    print(f"(args){args}")

    td = MquantTool(args.config_filename, args.date_change,
                 end_time, args.clear_atx, GATEWAY_NAME)

    td.start()



from datetime import datetime, timezone, timedelta
import os
import queue
import threading
import time
import json
import signal
import sys
import traceback
import argparse
import csv
import pathlib
import shutil
import pandas as pd

from pymongo import MongoClient, ASCENDING, DESCENDING

from ..mongo_client_gateway import MongoClientTradeGateway

from .constants import (GATEWAY_NAME, FILES)
from .utils import (decode_emc_status, side_to_target_type, decode_exchange_id, encode_emc_side,
                    decode_ordtype, encode_emc_market, get_today_date, get_time_from_str, decode_emc_market)


try:
    import thread
except ImportError:
    import _thread as thread

LL9 = 1000000000


class EmcServer(MongoClientTradeGateway):
    def __init__(self, config_filename, endtime, gateway_name):
        MongoClientTradeGateway.__init__(
            self, config_filename, endtime, gateway_name)

        self.load_tg_setting(config_filename)

        self.insert_orders = {}
        self.oid_to_ref = {}
        self.sids = []
        self.oid_to_traded = {}
        self.oid_to_traded_money = {}

        self.sid_to_req = {}
        self.oid_to_req = {}
        self.oid_to_mid = {}
        self.oid_to_acc = {}
        self.db_id_to_oid = {}
        self.ref_to_oid = {}
        # dbf文件当前index
        self.trade_count = {}
        for acc in self.accounts_run:
            self.trade_count[acc] = 0
        self.oid_to_local_ids = {}
        self.oid_to_instr_ids = {}

    def monitor(self):
        self.thread_pool.submit(self.monitor_algo_order_insert)
        self.thread_pool.submit(self.monitor_algo_task_update)
        # self.thread_pool.submit(self.monitor_cancel_order_insert)
        # self.thread_pool.submit(self.monitor_order_update)
        self.thread_pool.submit(self.monitor_trade_update)
        self.thread_pool.submit(self.update_asset)
        self.thread_pool.submit(self.update_equity_position)
        self.thread_pool.submit(self.date_change)

    def load_tg_setting(self, config_filename):
        try:
            #config_filename = os.path.join(config_path, 'atx_server_config.json')
            f = open(config_filename, encoding='gbk')
            setting = json.load(f)
            #self.insert_order_path = setting['insert_order_path'].replace('/', '\\')
            self.recv_msg_dir = setting['recv_msg_path'].replace('/', '\\')
            self.insert_order_msg_dir = setting['insert_order_msg_path'].replace(
                '/', '\\')
            self.pos_interval = setting['pos_interval']
            self.acc_interval = setting['acc_interval']
            self.algo_task_scan_interval = setting['algo_task_scan_interval']

        except Exception as e:
            err = traceback.format_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            traceback.print_exception(
                exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
            print(f"load config failed! (exception){err}")
            exit(0)

    @MongoClientTradeGateway.error_handler
    def req_buy_order_insert(self, obj):
        current_time = datetime.now()
        formatted_time = current_time.strftime("%Y%m%d%H%M%S%f")[:-3]
        temp_filename = self.insert_order_msg_dir + \
            '_temp\\' + FILES.ORDERS + '1_buy.' + formatted_time + '.csv'

        header = ['fundID', 'algoType', 'stkCode', 'Market', 'Direction', 'orderQty',
                  'orderPrice', 'beginTime', 'endTime', 'customerOrderRef', 'algoParam1', 'algoParam2']

        with open(temp_filename, 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(header)

            target_account_name = obj['accountName']
            acc = self.target_account_names_to_acc[target_account_name]
            account_id = self.account_id[acc]

            oid = str(self.gen_order_id())
            mid = str(obj['mid'])

            local_report_no = oid
            fund_id = int(account_id)
            stkcode = obj['ticker']
            market = encode_emc_market(stkcode)
            Direction = 1

            qty = int(obj["volume"])
            algo_type = decode_ordtype(obj['executionPlan']["order_type"])
            start_time = str(obj['executionPlan']["start_time"])
            end_time = str(obj['executionPlan']["end_time"])
            price = 0.0
            if not obj.get('price') is None:
                price = float(obj['price'])
            limit_action = 1
            if not obj.get('LimAction') is None:
                if not limit_action:
                    limit_action = 0
            after_action = 0
            participate = 10
            if not obj.get('participate') is None:
                participate = int(obj['participate'])
            algo = f"limit_action={limit_action}:after_action={after_action}"
            if algo_type == 'KFPOVCORE':
                algo += f':max_percentage={participate}:price={price}'

            row = [fund_id, algo_type, stkcode, market, Direction,
                   qty, '', start_time, end_time, oid, algo, 1]

            writer.writerow(row)

        # Save the workbook
        current_time = datetime.now()
        formatted_time = current_time.strftime("%Y%m%d%H%M%S%f")[:-3]
        final_filename = f"{self.insert_order_msg_dir}\\{FILES.ORDERS}1_buy.{formatted_time}.csv"
        shutil.move(temp_filename, final_filename)

        self.logger.info(
            f"[req_buy_order] insert_success (oid){oid} (mid){mid} (param){row}")
        db_id = obj['_id']
        order_dict = {
            'oid': oid,
            'mid': mid,
            "db_id": db_id,
            "ticker": obj['ticker'],
            "volume": qty,
            "accountName": target_account_name,
            "order_type": str(obj['executionPlan']["order_type"]),
            "start_time": str(obj['executionPlan']["start_time"]),
            "end_time": str(obj['executionPlan']["end_time"]),
            "order_time": datetime.utcnow(),
            'side': 1
        }

        target_collection = self.order_info_db[acc]['target']
        delete_query = {
            '_id': db_id
        }
        delete_res = target_collection.delete_one(delete_query)
        self.logger.info(f"[on_req_buy_order] delete (target){obj}")

        atx_order_collection = self.order_info_db[acc]['atx_order']
        local_id_list = []
        instr_id_list = []
        db_msg = {
            "oid": oid,
            "mid": mid,
            "traded_vol": 0,
            "traded_amt": 0,
            "local_ids": local_id_list,
            "instr_ids": instr_id_list,
            "order_msg": order_dict,
        }
        query = {"oid": oid}
        res = atx_order_collection.replace_one(query, db_msg, True)
        self.logger.info(
            f"[on_req_order_insert] insert_gf_order_info (res){res} (msg){db_msg}")

        self.sids.append(mid)

        self.sid_to_req[mid] = order_dict
        self.oid_to_req[oid] = order_dict
        self.oid_to_mid[oid] = mid
        self.db_id_to_oid[db_id] = oid
        self.oid_to_acc[oid] = acc
        self.oid_to_traded[oid] = 0
        self.oid_to_traded_money[oid] = 0
        self.oid_to_local_ids[oid] = local_id_list
        self.oid_to_instr_ids[oid] = instr_id_list

    @MongoClientTradeGateway.error_handler
    def req_sell_order_insert(self, obj):
        current_time = datetime.now()
        formatted_time = current_time.strftime("%Y%m%d%H%M%S%f")[:-3]
        temp_filename = self.insert_order_msg_dir + \
            '_temp\\' + FILES.ORDERS + '1_sell.' + formatted_time + '.csv'

        header = ['fundID', 'algoType', 'stkCode', 'Market', 'Direction', 'orderQty',
                  'orderPrice', 'beginTime', 'endTime', 'customerOrderRef', 'algoParam1', 'algoParam2']

        with open(temp_filename, 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(header)

            target_account_name = obj['accountName']
            acc = self.target_account_names_to_acc[target_account_name]
            account_id = self.account_id[acc]

            oid = str(self.gen_order_id())
            mid = str(obj['mid'])

            fund_id = int(account_id)
            stkcode = obj['ticker']
            market = encode_emc_market(stkcode)
            Direction = 2

            qty = int(obj["volume"])
            algo_type = decode_ordtype(obj['executionPlan']["order_type"])
            start_time = str(obj['executionPlan']["start_time"])
            end_time = str(obj['executionPlan']["end_time"])
            limit_action = 1
            if not obj.get('LimAction') is None:
                if not limit_action:
                    limit_action = 0

            after_action = 0
            price = 99999.99
            if not obj.get('price') is None:
                price = float(obj['price'])
            #algo_price = 99999.99
            participate = 10
            if not obj.get('participate') is None:
                participate = int(obj['participate'])
            algo = f"limit_action={limit_action}:after_action={after_action}"
            if algo_type == 'KFPOVCORE':
                algo += f':max_percentage={participate}:price={price}'
            row = [fund_id, algo_type, stkcode, market, Direction,
                   qty, '', start_time, end_time, oid, algo, 2]

            writer.writerow(row)

        # Save the workbook
        current_time = datetime.now()
        formatted_time = current_time.strftime("%Y%m%d%H%M%S%f")[:-3]
        final_filename = f"{self.insert_order_msg_dir}\\{FILES.ORDERS}1_buy.{formatted_time}.csv"
        shutil.move(temp_filename, final_filename)

        self.logger.info(
            f"[req_sell_order] insert_success (oid){oid} (mid){mid} (param){row}")
        db_id = obj['_id']
        order_dict = {
            'oid': oid,
            'mid': mid,
            "db_id": db_id,
            "ticker": obj['ticker'],
            "volume": qty,
            "accountName": target_account_name,
            "order_type": str(obj['executionPlan']["order_type"]),
            "start_time": str(obj['executionPlan']["start_time"]),
            "end_time": str(obj['executionPlan']["end_time"]),
            "order_time": datetime.utcnow(),
            'side': 2
        }

        target_collection = self.order_info_db[acc]['sell_target']
        delete_query = {
            '_id': db_id
        }
        delete_res = target_collection.delete_one(delete_query)
        self.logger.info(f"[on_req_sell_order] delete (target){obj}")

        atx_order_collection = self.order_info_db[acc]['atx_order']
        local_id_list = []
        instr_id_list = []
        db_msg = {
            "oid": oid,
            "mid": mid,
            "traded_vol": 0,
            "traded_amt": 0,
            "local_ids": local_id_list,
            "instr_ids": instr_id_list,
            "order_msg": order_dict,
        }
        query = {"oid": oid}
        res = atx_order_collection.replace_one(query, db_msg, True)
        self.logger.info(
            f"[on_req_order_insert] insert_{GATEWAY_NAME}_order_info (res){res} (msg){db_msg}")

        self.sids.append(mid)

        self.sid_to_req[mid] = order_dict
        self.oid_to_req[oid] = order_dict
        self.oid_to_mid[oid] = mid
        self.db_id_to_oid[db_id] = oid
        self.oid_to_acc[oid] = acc
        self.oid_to_traded[oid] = 0
        self.oid_to_traded_money[oid] = 0
        self.oid_to_local_ids[oid] = local_id_list
        self.oid_to_instr_ids[oid] = instr_id_list

    #@MongoClientTradeGateway.error_handler
    def monitor_algo_task_update(self):
        while not self.is_stopped:
            header = ['算法单号', '自定义编号', '资金账号', '证券代码', '证券名称', '市场', '委托方向', '算法类型', '状态',
                      '状态值', '目标数量', '执行进度', '成交数量', '成交金额', '成交均价', '开始时间', '结束时间', '备注', '算法风格']
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                algo_task_filename = self.recv_msg_dir + \
                    '\\' + str(trade_acc) + FILES.ALGO_TASKS + \
                    get_today_date() + '.csv'
                data_list = []
                algo_types = {
                    '算法单号': str,
                    '自定义编号': str
                }
                df = pd.DataFrame()
                try:
                    df = pd.read_csv(algo_task_filename,
                                     dtype=algo_types, encoding='gbk', on_bad_lines='skip')
                except Exception as e:
                    self.logger.warning(
                        f"[monitor_algo_task_update] file not exist error:{e}")
                for _, row in df.iterrows():
                    row_dict = {}
                    for col in header:
                        row_dict[col] = row[col]
                    data_list.append(row_dict)

                for record in data_list:
                    try:
                        if len(record['算法单号']) != 0 and len(record['自定义编号']) != 0:
                            oid = record['自定义编号']
                            ref = record['算法单号']
                            self.oid_to_ref[oid] = ref
                            self.ref_to_oid[ref] = oid
                            self.update_algo_status(record, acc)
                    except Exception as e:
                        self.logger.info(record)
                        self.logger.warning(e)
            time.sleep(self.algo_task_scan_interval)

    def update_algo_status(self, record, acc):

        if record['成交数量'] == 0 or len(record['算法单号']) == 0:
            self.logger.warning(
                f"[update_order] nothing_traded (record){record}")
            return

        mudan_id = record['算法单号']
        if not mudan_id in self.ref_to_oid:
            return
        oid = self.ref_to_oid[mudan_id]
        if len(oid) == 0:
            self.logger.warning(
                f"[update_order] not use file to order record:{record}")
            return

        mid = ""
        order_dict = {}
        if not oid in self.oid_to_acc:  # shut_down or other problem

            order_info_collection = self.order_info_db[acc]['atx_order']
            query = {'oid': oid}
            order_info_target = order_info_collection.find_one(query)
            if not order_info_target is None:
                order_dict = order_info_target['order_msg']
                mid = order_info_target['mid']
                try:
                    self.oid_to_instr_ids[oid] = order_info_target['instr_ids']
                except:
                    self.oid_to_instr_ids[oid] = []
                self.oid_to_traded_money[oid] = 0
                self.oid_to_traded[oid] = 0
                self.oid_to_mid[oid] = mid
                self.oid_to_req[oid] = order_dict
                self.oid_to_ref[oid] = mudan_id
            else:
                self.logger.error(f"[update_order] can't_find_oid (oid){oid}")
                return
        else:
            order_dict = self.oid_to_req[oid]
            mid = self.oid_to_mid[oid]

        traded_vol = record['成交数量']

        int_stock_code = int(record['证券代码'])
        ticker = f'{int_stock_code:06d}'
        exchange = decode_exchange_id(ticker)
        volume = record['成交数量']

        price = 0
        if (traded_vol != 0):
            price = float(record['成交均价'])

        tg_name = order_dict['accountName'].split('@')[0]
        accountName = order_dict['accountName'].split('@')[1]
        order_type = order_dict['order_type']
        start_time = order_dict['start_time']
        end_time = order_dict['end_time']
        order_time = order_dict['order_time']

        target_type = side_to_target_type(order_dict['side'])
        status = decode_emc_status(record['状态'])
        utc_start_time = get_time_from_str(str(start_time))

        replace_collection = self.tradelog_db[acc]['order']
        query = {'mid': mid, '_id': int(oid)}
        replace_target = replace_collection.find_one(query)
        dbtime = order_time if order_time > utc_start_time else utc_start_time

        order_msg = {
            "_id": int(oid),
            "tg_name": tg_name,  # 对应 account_info._id
            "exchange": exchange,  # 对应 tlclient.trader.constant 包 ExchangeID
            "target_type": target_type,    # 'buy' | 'sell'
            "volume": volume,  # 成交volume
            "price": price,  # 实际成交均价
            "order_type": order_type,
            "ticker": ticker,
            "mid": mid,  # target中对应的 母单sid
            "accountName": accountName,  # 对应 account_info.account_name
            "algo_args": {  # 具体的算法参数
                "order_type": order_type,
                "start_time": start_time,
                "end_time": end_time
            },
            "status": status,  # 'active' | 'filled' | 'canceled'
            "filled_vol": traded_vol,  # 实际成交的 volume
            "dbTime": dbtime,
        }

        res = replace_collection.replace_one(query, order_msg, True)
        self.logger.info(f"[rtn_order] (res){res} (order_msg){order_msg}")

        atx_order_collection = self.order_info_db[acc]['atx_order']
        atx_order_msg = {
            "instr_ids": self.oid_to_instr_ids[oid],
        }
        update_atx_order_msg = {"$set": atx_order_msg}
        query = {"oid": oid}
        res = atx_order_collection.update_one(query, update_atx_order_msg)
        self.logger.info(
            f"[update_order] update_atx_order_info (res){res} (msg){update_atx_order_msg}")

    @MongoClientTradeGateway.error_handler
    def monitor_order_update(self):
        while not self.is_stopped:
            header = ['算法单号', '资金账号', '证券代码', '委托日期', '委托时间', '委托价格', '委托编号', '合同序号',
                      '委托数量', '成交数量', '成交均价', '撤单数量', '委托状态', '委托状态值', '业务类型', '信用交易类型', '来源']
            algo_order_types = {
                '算法单号': str,
                '自定义编号': str
            }
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                order_filename = self.recv_msg_dir + \
                    '\\' + str(trade_acc) + FILES.ORDERS + \
                    get_today_date() + '.csv'

                data_list = []

                df = pd.DataFrame()
                try:
                    df = pd.read_csv(
                        order_filename, dtype=algo_order_types, encoding='gbk')
                except:
                    self.logger.warning(
                        "[monitor_order_update] file not exist")

                data_list = []
                current_len = df.shape[0]
                #order_count = self.order_count[acc]

                for _, row in df.iterrows():
                    row_dict = {}
                    for col in header:
                        row_dict[col] = row[col]
                    data_list.append(row_dict)

                for record in data_list:
                    if record['委托状态'] in ['已成', '部撤', '已撤']:
                        self.update_order(record, acc)
                    elif record['委托状态'] in ['废单']:
                        self.logger.info(
                            f"[monitor_order_update] order_rejected msg{record}")

            time.sleep(self.order_scan_interval)

    @MongoClientTradeGateway.error_handler
    def update_asset(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                asset_filename = self.recv_msg_dir + \
                    '\\' + str(trade_acc) + FILES.ACCOUNTS + \
                    get_today_date() + '.csv'
                df = pd.DataFrame()
                try:
                    df = pd.read_csv(asset_filename,  encoding='gbk')
                except:
                    self.logger.warning("[update_asset] file not exist")
                for _, row in df.iterrows():
                    avail_amt = row["可用资金"]
                    stkasset = row["总市值"]
                    all_asset = row["资金资产"]
                    ftmargin = row["资金余额"]  # 持仓占用保证金
                    creditmargin = row["可用资金"]  # 保证金可用
                    assurebalance = row["冻结资金"]  # 担保品可用

                    tg_name = self.tgnames[acc]
                    accountname = self.log_account_names[acc]
                    db_msg = {
                        "_id": tg_name,  # 对应之前的 TG 名称
                        "accountName": accountname,  # 产品名称
                        "avail_amt": avail_amt,  # 可用资金（可现金买入的资金）
                        # 净资产.  最好是由券商那边提供，通常普通账户是 可用资金(包含预扣费用) + 市值；信用账户是总资金 - 总负债
                        "balance": all_asset,
                        "holding": stkasset,  # 市值,
                        "updated_at": datetime.utcnow()
                    }

                    update_msg = {"$set": db_msg}
                    asset_collection = self.order_info_db[acc]['TestEquityAccount']
                    query = {'_id': tg_name, 'accountName': accountname}
                    res = asset_collection.update_one(query, update_msg)
                    self.logger.info(
                        f"[rtn_asset] (res){res} (asset_msg){db_msg}")

            time.sleep(self.acc_interval)

    @MongoClientTradeGateway.error_handler
    def update_equity_position(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                pos_filename = self.recv_msg_dir + \
                    '\\' + str(trade_acc) + FILES.POSITIONS + \
                    get_today_date() + '.csv'
                dtypes = {
                    '证券代码': str,
                    '市场': str,
                }
                df = pd.DataFrame()
                try:
                    df = pd.read_csv(
                        pos_filename, dtype=dtypes, encoding='gbk')
                except:
                    self.logger.warning(
                        "[update_equity_position] file not exist")
                for acc in self.accounts_run:
                    tg_position_collection = self.order_info_db[acc]['tg_equity_position']
                    remove = tg_position_collection.delete_many(
                        {'account_name': self.log_account_names[acc], 'tg_name': self.target_account_names[acc]})
                    self.logger.info(
                        f"[update_equity_position] delete_old_position_info (remove){remove} ")
                for _, row in df.iterrows():
                    market = row["市场"]
                    exchange = decode_emc_market(market)
                    stkcode = int(row["证券代码"])
                    ticker = f'{stkcode:06d}'
                    # direction = row["direction"] # 'B' 多 ‘S’ 空
                    out_direction = "long"

                    stkholdqty = row["持仓数量"]
                    stkavl = row["可用数量"]
                    costprice = row["成本价"]
                    #ftmargin = row["ftmargin"]

                    tg_name = self.target_account_names[acc]
                    accountname = self.log_account_names[acc]
                    pos_collection = self.order_info_db[acc]['tg_equity_position']
                    query = {'tg_name': tg_name,
                             'accountName': accountname, "ticker": ticker}
                    pos_msg = {
                        "account_name": accountname,
                        "tg_name": tg_name,
                        "ticker": ticker,
                        "exchange": exchange,
                        "direction": out_direction,  # long/short，没有short部分就只存long部分
                        "avail_pos": stkavl,  # 可用仓位
                        # 今仓，TODO 需要统计下不同券商盘中，对于卖出的position 是直接从 yd_pos 上减，还是在 td_pos 增加一个负的值。
                        "total_pos": stkholdqty,
                        "cost": costprice,  # 没有
                        "type": "stock",
                        "updated_at": datetime.utcnow()
                    }
                    res = pos_collection.replace_one(pos_msg, pos_msg, True)
                    self.logger.info(
                        f"[update_equity_position] (res){res.modified_count} (order_msg){pos_msg}")
            time.sleep(self.pos_interval)

    @MongoClientTradeGateway.error_handler
    def update_order(self, record, acc):
        if record['成交数量'] == 0 or len(record['算法单号']) == 0:
            self.logger.warning(
                f"[update_order] nothing_traded (record){record}")
            return
        mudan_id = record['算法单号']
        if not mudan_id in self.ref_to_oid:
            return
        oid = self.ref_to_oid[mudan_id]
        if len(oid) == 0:
            self.logger.warning(
                f"[update_order] not use file to order record:{record}")
            return

        mid = ""
        order_dict = {}
        if not oid in self.oid_to_acc:  # shut_down or other problem

            order_info_collection = self.order_info_db[acc]['atx_order']
            query = {'oid': oid}
            order_info_target = order_info_collection.find_one(query)
            if not order_info_target is None:
                order_dict = order_info_target['order_msg']
                mid = order_info_target['mid']
                try:
                    self.oid_to_instr_ids[oid] = order_info_target['instr_ids']
                except:
                    self.oid_to_instr_ids[oid] = []
                self.oid_to_traded_money[oid] = 0
                self.oid_to_traded[oid] = 0
                self.oid_to_mid[oid] = mid
                self.oid_to_req[oid] = order_dict
                self.oid_to_ref[oid] = mudan_id
            else:
                self.logger.error(f"[update_order] can't_find_oid (oid){oid}")
                return
        else:
            order_dict = self.oid_to_req[oid]
            mid = self.oid_to_mid[oid]

        traded_vol = record['成交数量']

        int_stock_code = int(record['证券代码'])
        ticker = f'{int_stock_code:06d}'
        exchange = decode_exchange_id(ticker)
        volume = record['成交数量']

        price = 0
        if (traded_vol != 0):
            price = float(record['成交价格'])

        tg_name = order_dict['accountName'].split('@')[0]
        accountName = order_dict['accountName'].split('@')[1]
        order_type = order_dict['order_type']
        start_time = order_dict['start_time']
        end_time = order_dict['end_time']
        order_time = order_dict['order_time']

        target_type = side_to_target_type(order_dict['side'])
        status = decode_emc_status(record['委托状态'])
        utc_start_time = get_time_from_str(str(start_time))

        replace_collection = self.tradelog_db[acc]['order']
        query = {'mid': mid, '_id': int(oid)}
        replace_target = replace_collection.find_one(query)
        dbtime = order_time if order_time > utc_start_time else utc_start_time

        order_msg = {
            "_id": int(oid),
            "tg_name": tg_name,  # 对应 account_info._id
            "exchange": exchange,  # 对应 tlclient.trader.constant 包 ExchangeID
            "target_type": target_type,    # 'buy' | 'sell'
            "volume": volume,  # 成交volume
            "price": price,  # 实际成交均价
            "order_type": order_type,
            "ticker": ticker,
            "mid": mid,  # target中对应的 母单sid
            "accountName": accountName,  # 对应 account_info.account_name
            "algo_args": {  # 具体的算法参数
                "order_type": order_type,
                "start_time": start_time,
                "end_time": end_time
            },
            "status": status,  # 'active' | 'filled' | 'canceled'
            "filled_vol": traded_vol,  # 实际成交的 volume
            "dbTime": dbtime,
        }

        res = replace_collection.replace_one(query, order_msg, True)
        self.logger.info(f"[rtn_order] (res){res} (order_msg){order_msg}")

        atx_order_collection = self.order_info_db[acc]['atx_order']
        atx_order_msg = {
            "instr_ids": self.oid_to_instr_ids[oid],
        }
        update_atx_order_msg = {"$set": atx_order_msg}
        query = {"oid": oid}
        res = atx_order_collection.update_one(query, update_atx_order_msg)
        self.logger.info(
            f"[update_order] update_atx_order_info (res){res} (msg){update_atx_order_msg}")

    @MongoClientTradeGateway.error_handler
    def monitor_algo_order_insert(self):
        while not self.is_stopped:
            print("[monitor_algo_order_insert]")
            for acc in self.accounts_run:
                target_account_name = self.target_account_names[acc]
                buy_query = {"accountName": target_account_name}
                buy_targets = self.order_info_db[acc]["target"].find(buy_query)
                if buy_targets.count() == 0:
                    self.logger.warning(
                        f"[monitor_buy_order] no_buy_target (acc){acc}")
                    continue
                for target in buy_targets:
                    if target['_id'] not in self.order_db_ids:
                        self.order_db_ids.append(target['_id'])
                        self.req_buy_order_insert(target)
                    else:
                        self.logger.warning(
                            f"[monitor_algo_order_insert] _id_existed (_id){target['_id']}")
            for acc in self.accounts_run:
                target_account_name = self.target_account_names[acc]
                sell_query = {"accountName": target_account_name}
                sell_targets = self.order_info_db[acc]["sell_target"].find(
                    sell_query)
                if sell_targets.count() == 0:
                    self.logger.warning(
                        f"[monitor_sell_order] no_sell_target (acc){acc}")
                    continue
                for sell_target in sell_targets:
                    if sell_target['_id'] not in self.sell_order_db_ids:
                        self.sell_order_db_ids.append(sell_target['_id'])
                        self.req_sell_order_insert(sell_target)
                    else:
                        self.logger.warning(
                            f"[monitor_algo_order_insert] _id_existed (_id){sell_target['_id']}")
            time.sleep(self.scan_interval)

    @MongoClientTradeGateway.error_handler
    def monitor_cancel_order_insert(self):
        while not self.is_stopped:
            for acc in self.accounts_run:
                print("[monitor_cancel_order_insert]")
                target_account_name = self.target_account_names[acc]
                query = {"accountName": target_account_name}
                with self.cancel_orderlock:
                    cancel_targets = self.order_info_db[acc]["cancel_target"].find(
                        query)
                    if cancel_targets.count() == 0:
                        self.logger.warning(
                            "[monitor_cancel_order] no_cancel_info")
                        continue
                    for cancel_target in cancel_targets:
                        if cancel_target['_id'] not in self.cancel_order_ids:
                            self.cancel_order_ids.append(cancel_target['_id'])
                            self.req_cancel_order(cancel_target, acc)
            time.sleep(self.scan_interval)

    @MongoClientTradeGateway.error_handler
    def monitor_trade_update(self):
        header = ['算法单号', '资金账号', '证券代码', '成交日期', '成交时间', '成交数量',
                  '委托编号', '成交编号', '成交价格', '回报类型', '业务类型', '信用交易类型', '来源']
        algo_trade_types = {
            '算法单号': str
        }
        while not self.is_stopped:
            print("[monitor_trade_update]")

            for acc in self.accounts_run:
                trade_acc = self.account_id[acc]
                trade_filename = self.recv_msg_dir + \
                    '\\' + str(trade_acc) + FILES.TRADES + \
                    get_today_date() + '.csv'
                data_list = []

                df = pd.DataFrame()
                try:
                    df = pd.read_csv(
                        trade_filename, dtype=algo_trade_types, encoding='gbk')
                except Exception as e:
                    self.logger.warning(
                        f"[monitor_trade_update] file not exist error:{e}")

                data_list = []

                for _, row in df.iterrows():
                    row_dict = {}
                    for col in header:
                        row_dict[col] = row[col]
                    data_list.append(row_dict)

                for record in data_list:
                    self.on_trade_msg(record, acc)

            time.sleep(self.trade_scan_interval)

    @MongoClientTradeGateway.error_handler
    def on_trade_msg(self, record, acc):
        if record['成交数量'] == 0:
            self.logger.warning(
                f"[on_trade_msg] nothing_traded (record){record}")
            return

        mudan_id = record['算法单号']

        if not mudan_id in self.ref_to_oid:
            self.logger.warning(
                f"[on_trade_msg] not use file to  trade record:{record}")
            return

        oid = self.ref_to_oid[mudan_id]

        mid = ""
        order_dict = {}
        if not oid in self.oid_to_acc:  # shut_down or other problem
            trade_acc = record['资金账号']
            acc = self.account_id_to_acc[trade_acc]
            order_info_collection = self.order_info_db[acc]['atx_order']
            query = {'oid': oid}
            order_info_target = order_info_collection.find_one(query)

            if not order_info_target is None:
                order_dict = order_info_target['order_msg']
                mid = order_info_target['mid']
                self.oid_to_local_ids[oid] = order_info_target['local_ids']
                self.oid_to_traded[oid] = 0
                self.oid_to_traded_money[oid] = 0
                self.oid_to_mid[oid] = mid
                self.oid_to_req[oid] = order_dict
                self.oid_to_ref[oid] = mudan_id
            else:
                self.logger.error(f"[on_trade_msg] can't_find_oid (oid){oid}")
                return
        else:
            acc = self.oid_to_acc[oid]
            order_dict = self.oid_to_req[oid]
            mid = self.oid_to_mid[oid]
        _id = str(int(record['成交编号']))
        if _id in self.oid_to_local_ids[oid]:
            #self.logger.warning(f"[on_trade_msg] duplicate_trade_msg (成交编号){_id}")
            return
        else:
            self.oid_to_local_ids[oid].append(_id)

        int_stock_code = int(record['证券代码'])
        ticker = f'{int_stock_code:06d}'
        exchange = decode_exchange_id(ticker)
        traded_vol = int(record['成交数量'])
        traded_price = float(record['成交价格'])
        trade_amt = float(traded_vol * traded_price)

        target_type = encode_emc_side(record['业务类型'])
        if target_type == 1:
            self.oid_to_traded_money[oid] += trade_amt
            self.oid_to_traded[oid] += traded_vol
        elif target_type == 2:
            self.oid_to_traded_money[oid] -= trade_amt
            self.oid_to_traded[oid] -= traded_vol
        entrust_vol = record['成交数量']
        #entrust_price = record.Price
        entrust_price = float(record['成交价格'])
        match_time = str(record['成交时间'])
        dbTime = get_time_from_str(match_time)

        commission = 0
        order_type = order_dict['order_type']
        target_account_name = order_dict['accountName']
        acc = self.target_account_names_to_acc[target_account_name]
        log_account_name = order_dict['accountName'].split('@')[1]
        tg_name = order_dict['accountName'].split('@')[0]

        side = encode_emc_side(record['业务类型'])
        trade_collection = self.tradelog_db[acc]['tg_trade']
        replace_trade_query = { "trade_ref": _id, "oid" : int(oid), "mid": mid, "accountName": log_account_name}
        trade_target = trade_collection.find_one(replace_trade_query)
        if not trade_target is None:
            dbTime = trade_target['dbTime']

        db_msg = {
            "trade_ref": _id,
            "oid": int(oid),
            "tg_name": tg_name,
            "exchange": exchange,
            "ticker": ticker,
            "traded_vol": traded_vol,
            "traded_price": traded_price,
            "order_type": order_type,
            "side": side,  # 对应 tlclient.trader.constant 包 Side
            "entrust_vol": entrust_vol,
            "entrust_price": entrust_price,  # 没有下单价格，只有成交价
            "dbTime": dbTime,
            "mid": mid,  # 对应订单中的 mid
            "commission": commission,  # 没有
            "trade_time": dbTime,  # 具体交易时间

            "accountName":  log_account_name,  # 对应 account_info.account_name
        }
        db_res = trade_collection.replace_one(
            replace_trade_query, db_msg, True)
        self.logger.info(
            f"[rtn_trade] (db_res){db_res} (db_msg){db_msg} (traded_vol){traded_vol} (traded_price){traded_price}")
        #self.update_position(traded_vol, order_dict, mid,oid, side, trade_amt, exchange)

        atx_order_collection = self.order_info_db[acc]['atx_order']
        atx_order_msg = {
            "local_ids": self.oid_to_local_ids[oid],
        }
        update_atx_order_msg = {"$set": atx_order_msg}
        query = {"oid": oid}
        res = atx_order_collection.update_one(query, update_atx_order_msg)
        self.logger.info(
            f"[on_trade_msg] update_atx_order_info (res){res} (msg){update_atx_order_msg}")

    @MongoClientTradeGateway.error_handler
    def update_position(self, traded_vol, order, mid, oid, side, trade_amt, exchange):
        target_account_name = order['accountName']
        acc = self.target_account_names_to_acc[target_account_name]

        query = {'mid': mid, 'accountName': target_account_name}
        position_collection = self.order_info_db[acc]['EquityPosition']
        position = position_collection.find_one(query)
        if position is None and traded_vol > 0:
            if oid not in self.oid_to_req:
                self.logger.warning(
                    f"[update_position] can't_find_req (mid){mid} (oid){oid}")
                return
            if oid not in self.oid_to_traded:
                self.logger.error(f"[update_position]no trade_vol (mid){mid}")
                return
            td_trade_vol = self.oid_to_traded[oid]

            amt = self.oid_to_traded_money[oid]

            ticker = order['ticker']
            yd_pos = 0
            mkt_value = amt
            cost = 0
            if not td_trade_vol == 0:
                cost = amt / td_trade_vol
            update_time = datetime.now(timezone.utc)

            position_msg = {
                "ticker": ticker,
                "cost": cost,  # 成本价
                "td_pos_long": td_trade_vol,  # 今仓
                "yd_pos_long": yd_pos,  # 昨仓（可卖），挂止盈单后为0
                "td_pos_short": 0,
                "yd_pos_short": 0,
                "actual_td_pos_long": td_trade_vol,  # 实际的今仓
                "actual_yd_pos_long": yd_pos,  # 实际的昨仓
                "enter_date": update_time,
                "holding_days": 0,  # 持仓时间（交易日累计）
                "mkt_value": mkt_value,  # 市值
                "exchange": exchange,  # 对应 tlclient.trader.constant 包 ExchangeID
                "mid": mid,  # 我们策略生成的id，需要根据实际下单情况维护各sid的持仓。
                # 格式: {account_info.tg_name}@{account_info.account_name}
                "accountName": target_account_name,
                "update_date": update_time,
            }
            query2 = {'mid': mid,
                      'accountName': target_account_name}
            res = position_collection.replace_one(query2, position_msg, True)
            self.logger.info(
                f"[update_position] (res){res} (msg){position_msg}")
        else:
            if oid not in self.oid_to_req:
                self.logger.warning(
                    f"[update_position] can't_find_req (mid){mid}")
                return
            if oid not in self.oid_to_traded:
                self.logger.error(f"[update_position]no trade_vol (mid){mid}")
                return
            actual_yd_pos = position['actual_yd_pos_long']
            amt = trade_amt
            yd_pos = position['yd_pos_long']
            if side == 1 or side == 3:
                td_trade_vol = position['actual_td_pos_long'] + traded_vol
                mkt_value = position['mkt_value'] + amt
            elif side == 2:
                td_trade_vol = position['actual_td_pos_long'] - traded_vol
                mkt_value = position['mkt_value'] - amt
                yd_pos = yd_pos - traded_vol

            ticker = order['ticker']

            cost = 0
            total_vol = td_trade_vol + yd_pos
            if not total_vol == 0:
                cost = mkt_value / total_vol
            update_time = datetime.now(timezone.utc)
            accountName = order['accountName']
            enter_time = position['enter_date']
            holding_days = position['holding_days']
            position_msg = {
                "ticker": ticker,
                "cost": cost,  # 成本价
                "td_pos_long": td_trade_vol,  # 今仓
                "yd_pos_long": yd_pos,  # 昨仓（可卖），挂止盈单后为0
                "td_pos_short": 0,
                "yd_pos_short": 0,
                "actual_td_pos_long": td_trade_vol,  # 实际的今仓
                "actual_yd_pos_long": actual_yd_pos,  # 实际的昨仓
                "enter_date": enter_time,
                "holding_days": holding_days,  # 持仓时间（交易日累计）
                "mkt_value": mkt_value,  # 市值
                "exchange": exchange,  # 对应 tlclient.trader.constant 包 ExchangeID
                "mid": mid,  # 我们策略生成的id，需要根据实际下单情况维护各sid的持仓。
                # 格式: {account_info.tg_name}@{account_info.account_name}
                "accountName": accountName,
                "update_date": update_time,
            }
            query = {'mid': mid,
                     'accountName': accountName}
            res = position_collection.replace_one(query, position_msg, True)
            self.logger.info(
                f"[update_position] (res){res} (msg){position_msg}")

    def start(self):
        signal.signal(signal.SIGTERM, self.signal_handler)
        signal.signal(signal.SIGINT, self.signal_handler)
        _msg = f"[login] {GATEWAY_NAME}_server_start (time){datetime.now()}"
        self.send_to_user(logger=self.logger, url_list=self.url_list, msg=_msg)
        self.monitor()

    def close(self):
        return super().close()

    def join(self):
        while self.is_stopped == False:
            time.sleep(0.01)
            if self.is_stopped:
                self.logger.info(
                    "[close] main thread is stopped,active_orders message will lose")
                self.close()


if __name__ == "__main__":

    sys.stdout.reconfigure(encoding='utf-8')
    description = "emc_server,get target from mongodb and serve gf"
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('-e', '--end_time', dest='end_time', default='15:30')
    _config_filename = "C:/Users/Administrator/Desktop/emc_batandconfig/emc_server_config.json"
    parser.add_argument('-p', '--config_filepath',
                        dest='config_filepath', default=_config_filename)

    args = parser.parse_args()
    print(f"(args){args}")

    td = EmcServer(args.config_filepath, args.end_time, GATEWAY_NAME)
    td.start()

    td.join()



from datetime import datetime
import sys
import os

from constants import ORDER_SIDE
import requests
import traceback
import json

def to_ft_flags(side):
    ft_flags = ""
    if side == ORDER_SIDE.BUY:
        ft_flags = "buy"
    elif side == ORDER_SIDE.SELL:
        ft_flags = "sell"

    return ft_flags


def decode_ft_flag(flag):
    side = 0
    if flag == 1:
        side = 1
    elif flag == 2:
        side = 2
    elif flag == 11:
        side = 3
    
    return side



def decode_exchange_id(int_exchange_id):
    exchange = 0
    if int_exchange_id == 3553:
        exchange = 101        #EXCHANGE_SSE 
    elif int_exchange_id == 3554:
        exchange = 102        #EXCHANGE_SZE 
    
    return exchange
    
def get_log_default_path():
    # python2: linux2, python3: linux
    if sys.platform.startswith("linux") or sys.platform == "darwin":
        dirs = "/shared/log"
    elif sys.platform == "win32":
        dirs = os.path.join(get_windows_first_disk() + "/tmp/linker/log")
    else:
        dirs = '.'

    return dirs

def get_today_date():
    return datetime.today().strftime('%Y-%m-%d')

def get_digit_from_env(env_name, default_num):
    num = str(os.environ.get(env_name))
    return int(num) if num.isdigit() else default_num

def get_log_given_path(path):
    dirs = os.path.join(path)
    return path

def send_to_user(logger, url_list, msg):
    try:
        if len(url_list) == 0:
            logger.info("[send_to_user] send_message_failed")
            return
        
        payload_message_feishu = {
            "msg_type": "text",
            "content": {
                "text": msg
            }
        }
        
        headers = {
            'Content-Type': 'application/json'
        }
        
        payload_message_dingding = {
            "msgtype": "text",
            "text": {"content": msg},
            "at": {
                "atMobiles": [""],
                "isAtAll": "false"  # @所有人 时为true，上面的atMobiles就失效了
            }
        }
        response = requests.request("POST", url_list[0], headers=headers, data=json.dumps(payload_message_feishu))
        data = response.json()
        logger.info(f"[send_to_user] (response){response} (data){data}")
        if len(url_list) > 1: 
            response_ding = requests.request("POST", url_list[1], headers=headers, data=json.dumps(payload_message_dingding))
            data_ding = response_ding.json()
            logger.info(f"[send_to_user] (response){response_ding} (data){data_ding}")
    except Exception as e:
        err = traceback.format_exc()
        exc_type, exc_value, exc_traceback = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
        logger.error(f'[send_to_user] send_message_failed(exception){err}')


# encoding: utf-8

from mquant_api import *
from mquant_struct import *
import json
import mmap,os
import datetime
import socket                               #导入socket模块
import threading
from trade_data_dump import TradeDataDownloader
from tcp_server import TcpServer

def initialize(context):
    fund_account_stock = context.get_fund_account_by_type(AccountType.normal)
    g.account_type_info = {}
    if fund_account_stock is not None and len(fund_account_stock) > 0:
        g.account_type_info[fund_account_stock] = AccountType.normal

    fund_account_margin = context.get_fund_account_by_type(AccountType.margin)
    if fund_account_margin is not None and len(fund_account_margin) > 0:
        g.account_type_info[fund_account_margin] = AccountType.margin

    g.max_symbol_cnt_per_algo_inst = 300
    g.download_trade_data = True

    if g.download_trade_data:
        g.trade_data_downloader = TradeDataDownloader()
        g.trade_data_downloader.set_context(context)
        g.trade_data_downloader.set_file_dir_path('C:/Users/SUST/Desktop/MQuantTradeData')
        g.trade_data_downloader.init_trade_files()

    tcp_server = TcpServer()
    tcp_server.start()
    
def strategy_params():
    """
    category:事件回调
    brief:策略运行参数定义
    desc: 策略运行参数定义，可选实现。策略可自定义运行参数，启动策略时，会在启动弹框中显示策略自定义的参数，客户在界面修改参数值，修改后的参数会写入到context对象的run_params字段内，客户可在策略程序中通过context对象获取策略运行参数。目前支持int、float、string、list、table、bool类型的参数。

    :return:dict对象，key为参数名，value为一个包含参数默认值、参数描述（选填）的字典
    :remark:可选实现，参数由策略自由填写，由策略平台解析显示在界面上，支持编辑框、下拉列表、勾选框三种形式的参数，后续可根据需求进一步丰富
    :example:
        dict_params = {
       '证券代码':{'value':'601688.SH/000002.SZ','desc':'策略交易的标的代码'},                    #'desc'字段可填写，也可不填写，编辑框类型参数
       '委买价格':{'value':'17.50/27.5'},                                                        #编辑框类型参数
       '委卖价格':{'value':'18.50/28.0'},                                                        #编辑框类型参数
       '补单价格档位':{'value':['最新价','对手方一档','对手方二档','对手方3档','涨停价','跌停价']},  #下拉列表格式参数
       '使用持仓':{'value': True, 'desc':'买入篮子时是否使用持仓中已有的成分券额度'},                #bool类型参数
       '撤单时间间隔':{'value':10}                                                                #编辑框类型参数
       }
        return json.dumps(dict_params)
    """
    pass

def on_strategy_start(context):
    """
    category:事件回调
    category-desc:MQuant的事件处理引擎会将交易系统内的各种事件以回调函数的形式通知给策略脚本，策略脚本根据自身的业务逻辑在对应的回调函数中进行相应处理
    brief:策略启动回调
    desc:策略启动回调，可选实现。策略初始化结束后，会立即调用此回调函数，策略在此回调函数中可以进行交易，不可读取外部文件、网络等。

    策略正式启动回调
    :param context:
    :return:
    :remark: 初始化函数返回后立即调用，在该回调函数中允许交易，不允许读取外部文件，可选实现
    """
    #get_position_info(context)

def handle_tick(context, tick, msg_type):
    """
    category:事件回调
    brief:TICK行情回调
    desc:TICK行情回调，可选实现。调用subscribe订阅TICK行情后，在此函数中接收实时TICK行情推送。回测模式下，如果选择TICK级回测，回测行情也在此回调函数中接收处理。

    实时行情接收函数
    :param context:
    :param tick: Tick对象
    :param msg_type 消息类型，暂不启用
    :return:
    :remark:可选实现
    """
    pass

def handle_data(context, kline_data):
    """
    category:事件回调
    brief:K线行情回调
    desc:K线行情回调，可选实现。调用subscribe订阅1分钟K线行情后，在此函数中接收实时的分钟K线行情推送。回测模式下，如果选择分钟K或者日K回测，回测行情也在此回调函数中接收处理。
    k线数据接收函数，包含实时分钟k，回测模式下的分钟k和日k
    :param context:
    :param kline_data:KLineDataPush类型，回测模式下分钟k和日k都通过该函数接收推送
    :return:
    :remark:可选实现
    """
    pass

def handle_order_record(context, order_record, msg_type):
    """
    category:事件回调
    brief:逐笔委托回调
    desc:逐笔委托回调，可选实现。调用subscribe订阅逐笔委托行情后，在此函数中接收实时的逐笔委托行情推送。回测模式不支持逐笔数据订阅。
    处理逐笔委托
    :param context: Context对象
    :param order_record: 逐笔委托数据，RecordOrder对象
    :param msg_type:
    :return:
    :remark:可选实现
    """
    pass

def handle_record_transaction(context, record_transaction, msg_type):
    """
    category:事件回调
    brief:逐笔成交回调
    desc:逐笔成交回调，可选实现。调用subscribe订阅逐笔成交行情后，在此函数中接收实时的逐笔成交行情推送。回测模式不支持逐笔数据订阅。
    处理逐笔成交
    :param context: Context对象
    :param record_transaction: 逐笔成交数据，RecordTransaction对象
    :param msg_type: 保留
    :return:
    :remark:可选实现
    """
    pass

def decode_ord_status(status):
    if status == OrderStatus.open:
        return 0
    elif status == OrderStatus.canceled:
        return 2
    elif status == OrderStatus.rejected:
        return 3
    elif status == OrderStatus.held:
        return 4

def transfer_order_side(side):
    if side == OrderSide.BUY:
        return 1
    elif side == OrderSide.SELL:
        return 2
    else:
        return -1

def deal_order_msg(ord):
    try:
        if g.trade_data_downloader is not None:
            g.trade_data_downloader.on_order_update(ord)
    except:
        pass

def handle_order_report(context, ord, msg_type):
    """
    category:事件回调
    brief:订单回报函数
    desc:订单回报函数，可选实现。策略报单后，只要订单状态改变，都会进入订单回报函数，策略可以在此函数中判断订单状态并决定后续操作，只要当前策略报单的订单回报会通过此函数通知给策略。回测模式下，订单回报也在此函数中处理。
    订单回报处理函数
    :param ord:Order对象
    :param msg_type 消息类型，透传字段，调用查询接口时传入可获得协程并发
    :return:
    :remark:可选实现
    """
    # print(ord.__dict__)
    deal_order_msg(ord)


def handle_execution_report(context, execution, msg_type):
    """
    category:事件回调
    brief:成交回报函数
    desc:成交回报函数，可选实现。策略报单后，只要订单产生成交（可能分多笔成交），都会进入成交回报函数，策略可以在此函数中获取本次成交信息（成交价格、数量、金额等），只要当前策略报单的成交回报会通过此函数通知给策略。回测模式下，成交回报也在此函数中处理。
    成交回报
    :param context:
    :param execution:Trade对象
    :param msg_type:
    :return:
    """
    try:
        if g.trade_data_downloader is not None:
            g.trade_data_downloader.on_execution_update(execution)
    except:
        pass

def on_fund_update(context, fund_info):
    """
    category:事件回调
    brief:资金推送
    desc:资金推送函数，可选实现。
    :param context:
    :param fund_info: FundUpdateInfo 类对象
    :return:
    """
    try:
        # print('recv fund update:', fund_info.__dict__)
        if g.trade_data_downloader is not None:
            g.trade_data_downloader.on_fund_update(fund_info)
    except:
        pass

def get_position_info(context):
    msg_type = "on_position"
    fund_account = context.get_fund_account_by_type(AccountType.normal)
    account_type = g.account_type_info[fund_account]
    pos_list = get_positions_ex(account_type)
    if pos_list is not None:
        for pos in pos_list:
            pos_dict = {
                "msg_type" : msg_type,
                "fund_account" : fund_account,
                "symbol" : pos.security,
                "total_amount" : pos.total_amount,
                "closeable_amount" : pos.closeable_amount,
                "init_amount" : pos.init_amount,
                "hold_cost" : pos.hold_cost,
                "value" : pos.value
            }

            send_tcp_msg(pos_dict)

def on_position_update(context, pos_info):
    """
    category:事件回调
    brief:持仓推送
    desc:持仓推送函数，可选实现。
    :param context:
    :param pos_info: Position类对象
    :return:
    """
    try:
        if g.trade_data_downloader is not None:
            g.trade_data_downloader.on_position_update(pos_info)
    except:
        pass

def handle_fund_flow(context, fund_flow):
    """
    category:事件回调
    brief:实时资金流向推送
    desc:实时资金流向推送函数，可选实现。
    :param context:
    :param fund_flow:资金流向数据，FundFlow类型
    :return:
    """
    pass

def on_strategy_params_change(params, path):
    """
    category:事件回调
    brief:参数修改回调
    desc:参数修改回调，可选实现。用户在策略执行监控界面的实例列表中，点击参数修改按钮修改实例参数时，参数信息会通过此回调通知给策略，支持修改策略启动参数值或者传入参数文件。

    监控界面修改策略实例参数回调
    :param params:参数，支持任意形式的文本参数
    :param path:如果传入的是参数文件，path为文件路径，否则path为空字符串
    :return:
    :remark:可选实现
    """
    pass

def on_rsp_user_params(context, json_params, wnd_title):
    """
    category:事件回调
    brief:请求用户参数异步响应
    desc:策略调用get_user_params在界面弹出一个参数框（异步模式），用户修改参数后，点击确定，修改后的参数将通过此回调函数回传给策略

    请求用户参数异步响应
    :param context:
    :param json_params:
    :param wnd_title:
    :return:
    :remark:可选实现
    """
    pass

def on_request_user_params_template(context, params):
    """
    category:事件回调
    brief:触发弹出用户参数设置窗口
    desc:用户在策略执行监控实例列表界面点击手工干预按钮，将会触发此回调函数，策略可以在此回调函数中获取用户参数，也可不实现
    :param context:
    :param params: 保留字段
    :return:
    :remark:可选实现
    """
    pass

def handle_etf_estimate_info(context, etf_estimate_info, msg_type):
    """
    实时ETF预估信息接收函数
    :param context:上下文
    :param etf_estimate_info: EtfEstimateInfo对象
    :param msg_type 保留字段
    :return:
    :remark:可选实现
    """
    pass

def market_open(context, trade_date):
    """
    category:事件回调
    brief:开盘信号
    desc:开盘信号，可选实现。回测模式下，回测周期内的每个交易日开始时，都会发送一个开盘信号给策略，策略在此回调函数中进行每日初始化操作。注意，仅回测模式有效。
    开盘信号，回测专用，在该回调中进行每日初始化操作，例如查询数据，处理静态数据等，日k回测在该回调函数中报单时，订单在当日撮合，在handle_data中报单时，订单在下一个交易日撮合
    :param context:
    :param trade_date:当前交易日
    :return:
    :remark:可选实现
    """
    pass

def market_close(context, trade_date):
    """
    category:事件回调
    brief:收盘信号
    desc:收盘信号，可选实现。回测模式下，回测周期内的每个交易日结束时，都会发送一个收盘信号给策略，策略在此回调函数中进行每日日终操作。注意，仅回测模式有效。
    收盘信号,回测专用
    :param context:
    :param trade_date: 交易日
    :return:
    :remark:可选实现
    """
    pass

def on_strategy_ready_stop(context):
    """
    category:事件回调
    brief:策略准备结束回调
    desc:策略准备结束回调，可选实现。停止策略时，会先调用此回调，调用结束后，再调用on_strategy_end回调。该回调中允许交易，不允许与外部系统交互，策略可选择在此回调函数中撤掉在途订单。
    :param context:
    :return:
    """
    pass

def on_strategy_end(context):
    """
    category:事件回调
    brief:策略结束回调
    desc:策略结束回调，可选实现。策略实例终止时，MQuant会调用此回调函数通知策略进行一些数据保存、环境清理等工作，待策略的处理工作结束后再结束策略进程。特别注意，如果直接关闭策略进程，可能导致此函数未执行完毕进程就已经关闭，建议通过策略执行监控界面或者通过stop_strategy API终止策略。
    策略结束时调用，用户可以在此函数中进行一些汇总分析、环境清理等工作
    :param context:
    :return:
    :remark:可选实现
    """
    pass

def on_recv_order_reply(context, order_reply):
    """
    category:事件回调
    brief:报撤单异步响应
    desc:报撤单异步响应，可选实现。必须调用set_enable_order_reply(True)开启响应消息推送后，才可收到响应。set_enable_order_reply是全局开关，任何一个实例调用此接口设置后，其他实例都会受影响。
    :param context:
    :param order_reply: class OrderReply
    :return:
    """
    pass

def on_system_msg(context, error_code, error_msg):
    """
    category:事件回调
    brief:系统消息
    desc:系统消息,可选实现。
    :param context:
    :param error_code:
    :param error_msg:
    :return:
    """
    pass



from datetime import datetime

#没有融券
def decode_cats_side(side):
    rtn_trade_side = 0
    if side == '1':
        rtn_trade_side = 1
    elif side == '2':
        rtn_trade_side = 2
    elif side == 'A':
        rtn_trade_side = 3
    return rtn_trade_side

def decode_cats_status(status, entrust_vol, filled_vol):
    new_status = ""
    if status in [0, 1]:
        new_status = 'active'
    elif status == 2:
        if filled_vol < entrust_vol:
            new_status = 'canceled'
        else:
            new_status = 'filled'
    return new_status

def side_to_target_type(side):
    target_type = ""
    if side in ['1','A']:
        target_type = "buy"
    elif side == '2':
        target_type = "sell"
    return target_type

def decode_exchange_id(str_exchange):
    exchange = 0
    if str_exchange == 'SH':
        exchange = 101        #EXCHANGE_SSE 
    elif str_exchange == 'SZ':
        exchange = 102        #EXCHANGE_SZE 
    return exchange


def decode_ordtype(order_type:str):
    ord_type = -1
    mapping = {
        'twap': 'TWAP',
        'vwap': 'VWAP',
        'vwap3': 'VWAP3',
        'twap3': 'TWAP3',
        'SmartVWAP' : 'SmartVWAP3',
        'SmartTWAP' : 'SmartTWAP3',
        'kf_vwap_plus' : 'KF_VWAP_Plus',
        'kf_twap_plus' : 'KF_TWAP_Plus',
        'kf_pov_core' : 'KF_POV_Core',
        'VolumeInline3' : 'VolumeInline3',
        'SmartVolumeInline3' : 'SmartVolumeInline3'
    }
    ord_type = mapping.get(order_type, -1)
    return ord_type



from datetime import datetime
import sys
import os

#没有融券
def decode_cats_side(side):
    rtn_trade_side = 0
    if side == '1':
        rtn_trade_side = 1
    elif side == '2':
        rtn_trade_side = 2
    elif side == 'A':
        rtn_trade_side = 3
    return rtn_trade_side

#0 new 2 filled 4 canceled 6 part canceled
def decode_cats_status(status):
    new_status = ""
    if status in [0, 1]:
        new_status = 'active'
    elif status == 2:
        new_status = 'filled'
    elif status == 4 or status == 3:
        new_status = 'canceled'
    return new_status

def side_to_target_type(side):
    target_type = ""
    if side in [1,5]:
        target_type = "buy"
    elif side == 2:
        target_type = "sell"
    return target_type

def decode_exchange_id(str_exchange):
    exchange = 0
    if str_exchange == 'SH' :
        exchange = 101        #EXCHANGE_SSE 
    elif str_exchange == 'SZ':
        exchange = 102        #EXCHANGE_SZE 
    return exchange


def decode_ordtype(order_type:str):
    ord_type = -1
    mapping = {
        'twap': 'TWAP',
        'vwap': 'VWAP',
        'pov': 'POV',
        'SmartTWAP': 'SmartTWAP',
        'SmartVWAP': 'SmartVWAP',
        'SmartPOV': 'SmartPOV'
    }
    ord_type = mapping.get(order_type, -1)
    return ord_type



class OrderMsg:
    def __init__(self):
        self.oid = ''
        self.sid = ''
        self.mudan_id = ''
        self.order_dict = {}
        self.traded_vol = 0
        self.traded_amt = 0.0
        self.local_ids = []

    def set_oid(self, oid):
        self.oid = oid

    def get_oid(self):
        return self.oid

    def set_sid(self, sid):
        self.sid = sid

    def get_sid(self):
        return self.sid

    def set_mudan_id(self, mudan_id):
        self.mudan_id = mudan_id

    def get_mudan_id(self):
        return self.mudan_id

    def set_order_dict(self, order_dict):
        self.order_dict = order_dict

    def get_order_dict(self):
        return self.order_dict

    def set_traded_vol(self, traded_vol):
        self.traded_vol = traded_vol

    def get_traded_vol(self):
        return self.traded_vol

    def set_traded_amt(self, traded_amt):
        self.traded_amt = traded_amt

    def get_traded_amt(self):
        return self.traded_amt

    def set_local_ids(self, local_ids):
        self.local_ids = local_ids

    def get_local_ids(self):
        return self.local_ids



import threading
import json
class TcpStringBuffer:
    """_summary_:用于tcp 收发的string buffer类
    """
    def __init__(self) -> None:
        self.recv_string_buffer = ''
        self.send_string_buffer = ''
        self.recv_str_lock = threading.Lock()
        self.send_str_lock = threading.Lock()

    def is_recv_str_empty(self):
        """
        Returns:
            bool: 1 or 0
        """
        return len(self.recv_string_buffer) == 0

    def is_send_str_empty(self):
        """_summary_: check send_list_empty

        """
        return len(self.send_string_buffer) == 0

    def receive(self, data:str):
        """_summary_:get data from socket recv
        Args:
            data (str): input_data
        """
        with self.recv_str_lock:
            self.recv_string_buffer += data
    def get_send(self):
        """send send_string_buffer
        """
        if len(self.send_string_buffer) == 0:
            return
        with self.send_str_lock:
            string_buffer = self.send_string_buffer
            self.send_string_buffer = ''
            return string_buffer
    def push(self, data:dict):
        """
        add  delimiter for tcp_send
        Args:
            data (str): send_data
        """
        new_data = json.dumps(data) + '\r\n'
        with self.send_str_lock:
            self.send_string_buffer += new_data

    def get_recv(self):
        """
        _summary_: 将str_buffer拆成dict_list,并清空
        """
        dict_list = []
        if len(self.recv_string_buffer) == 0:
            return
        with self.recv_str_lock:
            # 分割字符串并创建字典
            items = self.recv_string_buffer.split("\r\n")
            check_end = self.recv_string_buffer.endswith("\r\n")
            self.recv_string_buffer = ''
            if check_end:
                for item in items[:-1]:
                    try:
                        _dict = json.loads(item)
                        dict_list.append(_dict)
                    except:
                        print(f"error: item: {item}")
                self.recv_string_buffer = items[-1]
        return dict_list


import datetime
import socket                               #导入socket模块
import threading
import json
import time
from mquant_api import *
from mquant_struct import *
from string_buffer import TcpStringBuffer
class TcpServer():
    """_summary_: tcpserver for m quant
    """
    def __init__(self) -> None:
        self.tcp_socket_init = False
        self.tcp_socket = None
        self.conn = None
        self.msg_new = None
        self.addr = None
        self.string_buffer = TcpStringBuffer()
        self.host = '127.0.0.1'                       #主机IP
        self.port = 1210                             #端口
        self.is_stopped = False
        self.msg_lock = threading.Lock()
        self.send_msg_lock = threading.Lock()


    # 创建TCP服务
    def create_tcp_server(self):
        if self.tcp_socket_init is False:
            self.tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)                   #创建TCP/IP套接字
            self.tcp_socket.bind((self.host, self.port))        #绑定端口
            self.tcp_socket.listen(5)                           #设置最多连接数
            self.tcp_socket_init = True
        #阻塞模式
        print("等待TCP客户端连接") #建立客户端连接
        self.conn, self.addr = self.tcp_socket.accept()

    def tcp_proc(self):
        while not self.is_stopped:
            if self.tcp_socket_init is True:
                #获取客户端请求的数据,使用全局变量模式，供框架内其他函数使用
                msg_new = self.conn.recv(1024).decode()
                self.string_buffer.receive(msg_new)
        self.conn.close()

    def send_tcp_msg(self, msg):
        self.string_buffer.push(msg)
    #log.info("send_tcp_msg, msg_type:{} smg:{}".format(msg["msg_type"], msg))

    def msg_handler(self):
        while not self.is_stopped:
            if not self.string_buffer.is_recv_str_empty():
                for msg in self.string_buffer.get_recv():
                    self.handle_msg(msg)
            time.sleep(1)

    def send_msg_hander(self):
        while not self.is_stopped:
            if not self.string_buffer.is_send_str_empty():
                send_data = self.string_buffer.get_send()
                self.conn.sendall(send_data.encode())
                print(f'send tcp msg: {send_data}')
            time.sleep(1)

    def handle_msg(self, msg):
        msg_type = msg['msg_type']
        if msg_type == "req_order_insert":
            self.proc_split_order_algo_msg(msg)
        elif msg_type == "req_batch_order_insert":
            self.proc_split_order_algo_msg(msg, True)
        elif msg_type == "req_plain_order":
            self.req_plain_order(msg)
        elif msg_type == "req_limit_order":
            self.req_limit_order(msg)
        else:
            print("can not parse msg: {}".format(msg))
    
    def req_limit_order(self, msg):
        batch_no = str(msg['oid'])
        account_type = AccountType.margin
        if 'account_type' in msg['account_type']:
            if msg['account_type'] == 'normal':
                account_type = AccountType.normal
            elif msg['account_type'] == 'margin':
                account_type = AccountType.margin
        order_request = OrderRequest()
        side = msg['side']
        if side == 'buy':
            order_request.side = OrderSide.BUY
        elif side == 'sell':
            order_request.side = OrderSide.SELL
        else:
            log.error('参数错误，非法的订单方向：{}'.format(side))
            return
        order_request.symbol = msg['ticker']
        order_request.amount = int(msg['volume'])
        entrust_type = msg['entrust_type']
        if entrust_type == 6:
            order_request.entrust_type = EntrustType.creditFinancing
        elif entrust_type == 9:
            order_request.entrust_type = EntrustType.creditTransactions
        else:
            order_request.entrust_type = '0'
        limit_price = float(msg['price'])
        order_request.style = LimitOrderStyle(limit_price=limit_price)
        rsp = order_normal(order_request, account_type=account_type, batch_no=batch_no)
        if rsp is None:
            log.error("req plain order failed")
            return
        else:
            msg_dict = {
                "msg_type" : "on_rsp_plain_order_insert",
                "oid" : batch_no,
                "order_id" : rsp.order_id,
                "add_time" : rsp.add_time
            }

        self.send_tcp_msg(msg_dict)

    def req_plain_order(self, msg):
        batch_no = str(msg['oid'])
        order_request = OrderRequest()
        side = msg['side']
        if side == '1':
            order_request.side = OrderSide.BUY
        elif side == '2':
            order_request.side = OrderSide.SELL
        else:
            log.error('参数错误，非法的订单方向：{}'.format(side))
            return
        order_request.symbol = msg['ticker']
        order_request.amount = int(msg['volume'])
        limit_price = self.get_limit_price(msg['ticker'], side)
        order_request.style = MarketOrderStyle('a', limit_price=limit_price)
        rsp = order_normal(order_request, batch_no=batch_no)
        if rsp is None:
            log.error("req plain order failed")
            return
        else:
            msg_dict = {
                "msg_type" : "on_rsp_plain_order_insert",
                "oid" : batch_no,
                "order_id" : rsp.order_id,
                "add_time" : rsp.add_time
            }

        self.send_tcp_msg(msg_dict)
        
    def proc_split_t0_algo_msg(self, params):
        """
        处理T0算法创建消息
        """
        log.info('recv algo params {}'.format(params))

    def proc_split_order_algo_msg(self, params, batch=False):
        """
        处理算法创建消息
        """
        log.info('recv algo params {}'.format(params))
        fund_account = params['fund_account']
        split_algo_param = SplitOrderAlgoParam()
        algo_type = params['algo_type']
        if params['algo_type'] == 27:
            algo_type = AlgoType.KF_TWAPPLUS
        elif params['algo_type'] == 28:
            algo_type = AlgoType.KF_VWAPPLUS
        elif params['algo_type'] == 29:
            algo_type = AlgoType.smarT_I
        elif params['algo_type'] == 30:
            algo_type = AlgoType.smarT_Y
        try:
            split_algo_param.algo_type = algo_type 
        except Exception as e:
            err_msg = 'algo_type insert failed'
            log.error(f'{err_msg}')
            return

        try:
            split_algo_param.start_time = datetime.datetime.strptime(params['start_time'], '%Y-%m-%d %H:%M:%S')
        except Exception as e:
            log.error('开始时间[{}]格式非法,请按照yyyy-MM-dd hh:mm:ss格式填写'.format(params['start_time']))
            return

        try:
            split_algo_param.end_time = datetime.datetime.strptime(params['end_time'], '%Y-%m-%d %H:%M:%S')
        except Exception as e:
            log.error('结束时间[{}]格式非法,请按照yyyy-MM-dd hh:mm:ss格式填写'.format(params['end_time']))
            return

        side = params['side']
        if side == '1':
            split_algo_param.order_side = OrderSide.BUY
        elif side == '2':
            split_algo_param.order_side = OrderSide.SELL
        else:
            split_algo_param.order_side = OrderSide.UNKNOWN
        entrust_type = params['entrust_type']
        if entrust_type == 6:
            split_algo_param.entrust_type = EntrustType.creditFinancing
        elif entrust_type == 9:
            split_algo_param.entrust_type = EntrustType.creditTransactions
        else:
            split_algo_param.entrust_type = '0'
        account_type = AccountType.margin
        if 'account_type' in params['account_type']:
            if params['account_type'] == 'normal':
                account_type = AccountType.normal
            elif params['account_type'] == 'margin':
                account_type = AccountType.margin
        

        split_algo_param.remark = str(params['oid'])
        
        split_algo_param.limit_price = params['limit']
        if algo_type == AlgoType.KF_TWAPPLUS or algo_type == AlgoType.KF_VWAPPLUS:
            split_algo_param.up_limit = params['up_limit'] # 涨幅限制 涨幅限制，单位为%，范围[-1,20]，小数位数不限制。填-1或者为空时，相当于此参数不使用；
            split_algo_param.down_limit = params['down_limit']  # 跌幅限制 跌幅限制，单位为%，范围[-1,20]，小数位数不限制。填-1或者为空时，相当于此参数不使用；
            split_algo_param.limit_action = params['limit_action']  # 涨跌停是否继续交易 True:涨停可以卖、跌停可以买 False:涨停不卖、跌停不买
            split_algo_param.after_action = params['after_action']  # 过期后是否继续交易 True:到了结束时间，算法未完成全部交易，继续交易，直到闭市，不参与收盘集合竞价
        elif algo_type == AlgoType.smarT_I:
            split_algo_param.expiration_date = datetime.datetime.strptime(params['expire_date'], '%Y-%m-%d').date()
        elif algo_type == AlgoType.smarT_Y:
            split_algo_param.expiration_date = datetime.datetime.strptime(params['expire_date'], '%Y-%m-%d').date()
            split_algo_param.occupy_amount = params['occupy_amount']
        oid_list = []
        if batch is True:
            algo_info_params = params['param_list']
            for param in algo_info_params:
                symbol = param['stk_id']
                qty = int(param['volume'])
                s_oid = str(param['oid'])
                oid_list.append(s_oid)
                order_item = AlgoOrderInfo()
                order_item.symbol = symbol
                order_item.amount = qty
                order_item.remark = s_oid
                order_item.buy_entrust_type = EntrustType.entrust  # 买入类型
                order_item.sell_entrust_type = EntrustType.entrust
                split_algo_param.order_list.append(order_item)
        else:
            symbol = params['ticker']
            qty = int(params['volume'])
            limit = str(params['limit'])

            order_item = AlgoOrderInfo()
            order_item.symbol = symbol
            order_item.amount = qty
            order_item.limit = limit
            split_algo_param.order_list.append(order_item)

        print('create algo instance')
        rsp = AlgoTradeHandler.start_split_order_algo_instance(account_type, split_algo_param)
        log.info('创建算法[{}]返回：{},{}'.format(algo_type, rsp.inst_id, rsp.err_info))
        rsp_msg = {
            "msg_type" : "on_rsp_order_insert",
            "status" : True,
            "inst_id" : rsp.inst_id,
            "oid": str(params['oid']),
            "err_info" : rsp.err_info
        }
        if batch is True:
            rsp_msg['oid_list'] = oid_list

        self.send_tcp_msg(rsp_msg)
    def get_limit_price(self, symbol, side):
        symbol_detail = get_symbol_detial(symbol)
        limit_price = 1.0
        if side == '1':
            limit_price = symbol_detail.HighLimitPrice
        elif side == '2':
            limit_price = symbol_detail.LowLimitPrice
        return limit_price
    def start(self):
        self.create_tcp_server()
        self.run()
    def run(self):
        ts = [
            threading.Thread(target=self.tcp_proc),
            threading.Thread(target=self.send_msg_hander),
            threading.Thread(target=self.msg_handler),
        ]
        for t in ts:
            t.setDaemon(True)
            t.start()
    def close(self):
        self.is_stopped = True



import sys
import argparse
import traceback
import json
from datetime import datetime

from mongo_client_gateway import MongoClientTradeGateway
from constants import (ENCODING, FILES, GATEWAY_NAME, Exchange, OrderStatus, OrderType, SecurityType, Side)

class AtxTool(MongoClientTradeGateway):
    def __init__(self, config_filename, sync_pos,  endtime):
        MongoClientTradeGateway.__init__(self, config_filename, endtime)
        self.update_trade_date = self.date
        self.sync_pos = sync_pos
        self.load_tool_setting(config_filename)

    def load_tool_setting(self, config_filename):
        try:
            #config_filename = os.path.join(config_path, 'atx_server_config.json')
            f = open(config_filename, encoding='gbk')
            setting = json.load(f)
            #self.recv_msg_dir = setting['recv_msg_path'].replace('/', '\\')
            self.sync_position_open = setting['sync_position_open']
            self.sync_position_close = setting['sync_position_close']

        except Exception as e:
            err = traceback.format_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            traceback.print_exception(exc_type, exc_value, exc_traceback, limit=None, file=sys.stdout)
            print(f"load config failed! (exception){err}")
            exit(0)
    
    def start(self):
        
        if self.sync_pos == True:
            self.date_change()
            _msg = "[atx_tool_date_change] executed"
            self.send_to_user(logger=self.logger, url_list=self.url_list, msg=_msg)
    
    @MongoClientTradeGateway.error_handler
    def date_change(self):
        dt = datetime.now()
        self.logger.info(f"(dt){dt.hour}")
        if dt.hour <= self.sync_position_open and self.sync_pos == True:
            self.logger.info("[date_change] date_change_open")
            self.update_position_date_open()
        elif dt.hour < self.sync_position_close and dt.hour > self.sync_position_open:
            self.logger.info("[date_change] date_not_change")
        elif dt.hour >= self.sync_position_close and self.sync_pos == True:
            self.logger.info("[date_change] date_change_close")
            self.update_position_date_close()
        else:
            self.logger.info("[date_change] date_not_change")
            return


    def update_position_date_open(self):
        for acc in self.accounts_run:
            collection = self.order_info_db[acc]['EquityPosition']
            target_account_name = self.target_account_names[acc]
            targets_query = {'accountName' : target_account_name}
            targets = collection.find(targets_query)
            self.logger.info(f"now (target_account_name){target_account_name}")
            if targets.count() == 0:
                continue
            for position in targets:
                        self.logger.info(f"goes in (position){position}")
                        mid = position['mid']
                        query = {'mid' : mid, 'accountName' : target_account_name}
                        holdingdays = position['holding_days']
                        yd_pos = position['actual_td_pos_long'] + position['actual_yd_pos_long']
                        td_pos = 0
                        change = {
                                        'holding_days': holdingdays,
                                        'yd_pos_long': yd_pos,
                                        'td_pos_long': td_pos,
                                        'actual_yd_pos_long': yd_pos,
                                        'actual_td_pos_long': td_pos
                                    }
                        new_data = {"$set": change}
                        res = collection.update_one(
                                                query, new_data, True)
                        self.logger.info(f"[date_change_open] (res){res} (change){change}")
    
    def update_position_date_close(self):
        for acc in self.accounts_run:
            collection = self.order_info_db[acc]['EquityPosition']
            target_account_name = self.target_account_names[acc]
            targets_query = {'accountName' : target_account_name}
            targets = collection.find(targets_query)
            if targets.count() == 0:
                continue
            for position in targets:
                    mid = position['mid']
                    query = {'mid' : mid, 'accountName': target_account_name}
                    holding_days = position['holding_days'] + 1
                    yd_pos = position['actual_td_pos_long'] + position['actual_yd_pos_long']
                    td_pos = 0
                            
                    change = {
                                        'holding_days': holding_days,
                                        'yd_pos_long': yd_pos,
                                        'td_pos_long': td_pos,
                                        'actual_yd_pos_long': yd_pos,
                                        'actual_td_pos_long': td_pos
                                    }
                    new_data = {"$set": change}
                    res = collection.update_one(
                                                query, new_data, True)
                    self.logger.info(f"[date_change_close] (res){res} (mid){mid} (change){change}")

def str2bool(v):
    if isinstance(v, bool):
        return v
    if v.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.') 

if __name__ == "__main__":

    description = "sync pos for atx_server"
    parser = argparse.ArgumentParser(description=description)

    _file_name = "C:\\Users\\Administrator\\Desktop\\atx_batandconfig\\atx_tool_setting.json"
    _date = datetime.now().strftime("%Y%m%d")
    parser.add_argument('-m', '--config_filename', dest='config_filename', type=str, default=_file_name)
    parser.add_argument('-p', '--date_change', dest='date_change', type=str2bool, default='T')
    end_time = "19:00"
    args = parser.parse_args()
    print (f"(args){args}")

    td = AtxTool(args.config_filename, args.date_change,end_time)

    td.start()